@@ -130,11 +130,12 @@ int dt_exif_read(dt_image_t *img, const char* path)
     {
       img->exif_aperture = pos->toFloat ();
     }
-    /* Read ISO speed */
+    /* Read ISO speed - Nikon happens to return a pair for Lo and Hi modes */
     if ( (pos=Exiv2::isoSpeed(exifData) )
          != exifData.end() )
     {
-      img->exif_iso = pos->toFloat ();
+      int isofield = pos->count () > 1  ? 1 : 0;
+      img->exif_iso = pos->toFloat (isofield);
     }
 #if EXIV2_MINOR_VERSION>19
     /* Read focal length  */
@@ -906,6 +907,7 @@ int dt_exif_xmp_read (dt_image_t *img, const char* filename, const int history_o
           
           sqlite3_step (stmt_upd_hist);
           free(params);
+          free(blendop_params);
 
           sqlite3_reset(stmt_sel_num);
           sqlite3_clear_bindings(stmt_sel_num);
@@ -1113,6 +1115,7 @@ void dt_exif_init()
   // Exiv2::LogMsg::setLevel(Exiv2::LogMsg::error);
 
   Exiv2::XmpParser::initialize();
+  // this has te stay with the old url (namespace already propagated outside dt)
   Exiv2::XmpProperties::registerNs("http://darktable.sf.net/", "darktable");
 }
 
