@@ -56,10 +56,9 @@ void dt_history_delete_on_image(int32_t imgid)
 
   dt_image_cache_release (img, 'r');
 
-  /* remove darktable|changed tag */
-  guint tagid = 0;
-  dt_tag_new("darktable|changed",&tagid);
-  dt_tag_detach(tagid, imgid);
+  /* remove darktable|style|* tags */
+  dt_tag_detach_by_string("darktable|style%",imgid);
+
 }
 
 void
