@@ -17,6 +17,7 @@
 */
 #include "develop/pixelpipe.h"
 #include "develop/blend.h"
+#include "develop/tiling.h"
 #include "gui/gtk.h"
 #include "control/control.h"
 #include "common/opencl.h"
@@ -34,6 +35,8 @@
 // this is to ensure compatibility with pixelpipe_gegl.c, which does not need to build the other module:
 #include "develop/pixelpipe_cache.c"
 
+#define max(a,b) ((a) > (b) ? (a) : (b))
+
 int dt_dev_pixelpipe_init_export(dt_dev_pixelpipe_t *pipe, int32_t width, int32_t height)
 {
   int res = dt_dev_pixelpipe_init_cached(pipe, 4*sizeof(float)*width*height, 2);
@@ -471,81 +474,137 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
     }
     dt_times_t start;
     dt_get_times(&start);
+
+    dt_develop_tiling_t tiling = { 0 };
+
+    /* get tiling requirement of module */
+    module->tiling_callback(module, piece, &roi_in, roi_out, &tiling);
+
+    assert(tiling.factor > 0.0f && tiling.factor < 100.0f);      
+
 #ifdef HAVE_OPENCL
-    if (dt_opencl_is_inited())
+    /* do we have opencl at all? did user tell us to use it? did we get a resource? */
+    if (dt_opencl_is_inited() && pipe->opencl_enabled && pipe->devid >= 0)
     {
       int success_opencl = TRUE;
-      int valid_input_on_gpu = FALSE;
+
+      /* if input is on gpu memory only, remember this fact to later take appropriate action */
+      int valid_input_on_gpu_only = (cl_mem_input != NULL);
       
-      /* general remark: in case of opencl errors within modules of out-of-memory on GPU, we always transparently
-         fall back to the respective cpu module and continue in pixelpipe. If we encounter fatal errors, we set 
-         pipe->opencl_error, return with value 1, and leave appropriate action to the calling function */
+      /* general remark: in case of opencl errors within modules or out-of-memory on GPU, we transparently
+         fall back to the respective cpu module and continue in pixelpipe. If we encounter late errors(*), we set 
+         pipe->opencl_error=1, return this function with value 1, and leave appropriate action to the calling function,
+         which normally would restart pixelpipe without opencl.
+         (*) late errors are sometimes detected when trying to get back data from device into host memory */
 
-      /* try to run opencl module after checking some pre-requisites */
-      if(pipe->opencl_enabled && module->process_cl && piece->process_cl_ready  && 
-         dt_opencl_image_fits_device(pipe->devid, roi_in.width, roi_in.height) && 
-         dt_opencl_image_fits_device(pipe->devid, roi_out->width, roi_out->height))
+      /* try to enter opencl path after checking some module specific pre-requisites */
+      if(module->process_cl && piece->process_cl_ready)
       {
-        // fprintf(stderr, "[opencl_pixelpipe 1] for module `%s', have bufs %lX and %lX \n", module->op, (long int)cl_mem_input, (long int)*cl_mem_output);
 
-        // if input is on the gpu only, remember this fact to later take appropriate action,
-        // else, if input is not on the gpu, copy it there.
-        if(cl_mem_input != NULL)
+        // fprintf(stderr, "[opencl_pixelpipe 0] factor %f, overhead %d, width %d, height %d, bpp %d\n", (double)tiling.factor, tiling.overhead, roi_in.width, roi_in.height, bpp);
+
+        // fprintf(stderr, "[opencl_pixelpipe 1] for module `%s', have bufs %lX and %lX \n", module->op, (long int)cl_mem_input, (long int)*cl_mem_output);
+        // fprintf(stderr, "[opencl_pixelpipe 1] module '%s'\n", module->op);
+
+        if(dt_opencl_image_fits_device(pipe->devid, max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), max(in_bpp, bpp), tiling.factor, tiling.overhead))
         {
-          /* remember that we found a valid input buffer on gpu */
-          valid_input_on_gpu = TRUE;
+          /* image is small enough -> try to directly process entire image with opencl */
+
+          // fprintf(stderr, "[opencl_pixelpipe 2] module '%s' running directly with process_cl\n", module->op);
+
+          /* input is not on gpu memory -> copy it there */
+          if (cl_mem_input == NULL)
+          {
+            cl_mem_input = dt_opencl_copy_host_to_device(pipe->devid, input, roi_in.width, roi_in.height, in_bpp);
+            if (cl_mem_input == NULL)
+            {
+              dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] couldn't generate input buffer for module %s\n", module->op);
+              success_opencl = FALSE;
+            }
+          }
+
+          /* try to allocate GPU memory for output */
+          if (success_opencl)
+          {
+            *cl_mem_output = dt_opencl_alloc_device(pipe->devid, roi_out->width, roi_out->height, bpp);
+            if (*cl_mem_output == NULL)
+            {
+              dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] couldn't allocate output buffer for module %s\n", module->op);
+              success_opencl = FALSE;
+            }
+          }
+
+          // fprintf(stderr, "[opencl_pixelpipe 2] for module `%s', have bufs %lX and %lX \n", module->op, (long int)cl_mem_input, (long int)*cl_mem_output);
+
+          /* now call process_cl of module; module should emit meaningful messages in case of error */
+          if (success_opencl)
+            success_opencl = module->process_cl(module, piece, cl_mem_input, *cl_mem_output, &roi_in, roi_out);
+
+
+          /* process blending */
+          if (success_opencl)
+            success_opencl = dt_develop_blend_process_cl(module, piece, cl_mem_input, *cl_mem_output, &roi_in, roi_out);
+        }
+        else if(module->flags() & IOP_FLAGS_ALLOW_TILING)
+        {
+          /* image is too big for direct opencl processing -> try to process image via tiling */
+
+          // fprintf(stderr, "[opencl_pixelpipe 3] module '%s' tiling with process_tiling_cl\n", module->op);
+
+          /* we might need to copy back valid image from device to host */
+          if (cl_mem_input != NULL)
+          {
+            cl_int err;
+
+            /* copy back to CPU buffer, then clean unneeded buffer */
+            err = dt_opencl_copy_device_to_host(pipe->devid, input, cl_mem_input, roi_in.width, roi_in.height, in_bpp);
+            if (err != CL_SUCCESS)
+            {
+              /* late opencl error */
+              dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (a)] late opencl error detected while copying back to cpu buffer: %d\n", err);
+              dt_opencl_release_mem_object(cl_mem_input);
+              pipe->opencl_error = 1;
+              dt_pthread_mutex_unlock(&pipe->busy_mutex);
+              return 1;
+            }
+            dt_opencl_release_mem_object(cl_mem_input);
+            cl_mem_input = NULL;
+            valid_input_on_gpu_only = FALSE;
+          }
+
+          /* now call process_tiling_cl of module; module should emit meaningful messages in case of error */
+          if (success_opencl)
+            success_opencl = module->process_tiling_cl(module, piece, input, *output, &roi_in, roi_out, in_bpp);
+
+          /* do process blending on cpu (this is anyhow fast enough) */
+          if (success_opencl)
+            dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
         }
         else
         {
-          cl_mem_input = dt_opencl_copy_host_to_device(pipe->devid, input, roi_in.width, roi_in.height, in_bpp);
-          if (cl_mem_input == NULL)
-          {
-            dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] couldn't generate input buffer for module %s\n", module->op);
-            success_opencl = FALSE;
-          }
+          /* image is too big for direct opencl and tiling is not allowed -> no opencl processing for this module */
+          success_opencl = FALSE;
         }
 
-        /* try to allocate GPU memory for output */
-        if (success_opencl)
-        {
-          *cl_mem_output = dt_opencl_alloc_device(pipe->devid, roi_out->width, roi_out->height, bpp);
-          if (*cl_mem_output == NULL)
-          {
-            dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] couldn't allocate output buffer for module %s\n", module->op);
-            success_opencl = FALSE;
-          }
-        }
-
-        // fprintf(stderr, "[opencl_pixelpipe 2] for module `%s', have bufs %lX and %lX \n", module->op, (long int)cl_mem_input, (long int)*cl_mem_output);
-
-        /* now process opencl module; modules should emit meaningful messages in case of error */
-        if (success_opencl)
-          success_opencl = module->process_cl(module, piece, cl_mem_input, *cl_mem_output, &roi_in, roi_out);
-
-
-        /* next process blending */
-        if (success_opencl)
-          success_opencl = dt_develop_blend_process_cl(module, piece, cl_mem_input, *cl_mem_output, &roi_in, roi_out);
-
+        
         // if (rand() % 20 == 0) success_opencl = FALSE; // Test code: simulate spurious failures
 
-        /* Finally check, if we were successful */
+        /* finally check, if we were successful */
         if (success_opencl)
         {
           /* Nice, everything went fine */
-          
-          /* input was anyhow only on GPU? Let's invalidate CPU input buffer then */
-          if (valid_input_on_gpu) dt_dev_pixelpipe_cache_invalidate(&(pipe->cache), input);
-
+        
           /* we can now release cl_mem_input */
-          dt_opencl_release_mem_object(cl_mem_input);
+          if (cl_mem_input) dt_opencl_release_mem_object(cl_mem_input);
           // we speculate on the next plug-in to possibly copy back cl_mem_output to output,
           // so we're not just yet invalidating the (empty) output cache line.
         }
         else
         {
           /* Bad luck, opencl failed. Let's clean up and fall back to cpu module */
-          dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] failed to run module %s. fall back to cpu module\n", module->op);
+          dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe] failed to run module '%s'. fall back to cpu path\n", module->op);
+
+          // fprintf(stderr, "[opencl_pixelpipe 4] module '%s' running on cpu\n", module->op);
 
           /* we might need to free unused output buffer */
           if (*cl_mem_output != NULL)
@@ -554,27 +613,36 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
             *cl_mem_output = NULL;
           }
           
-          /* first check where we found input buffer before we started */
-          if (valid_input_on_gpu)
+          /* check where our input buffer is located */
+          if (cl_mem_input != NULL)
           {
             cl_int err;
 
-            /* copy back to CPU buffer, then clean unneeded buffer */
+            /* copy back to host memory, then clean no longer needed opencl buffer.
+               important info: in order to make this possible, opencl modules must
+               not spoil their input buffer, even in case of errors. */
             err = dt_opencl_copy_device_to_host(pipe->devid, input, cl_mem_input, roi_in.width, roi_in.height, in_bpp);
             if (err != CL_SUCCESS)
             {
-              /* fatal opencl error */
-              dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (a)] fatal opencl error while copying back to cpu buffer: %d\n", err);
+              /* late opencl error */
+              dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (b)] late opencl error detected while copying back to cpu buffer: %d\n", err);
               dt_opencl_release_mem_object(cl_mem_input);
               pipe->opencl_error = 1;
               dt_pthread_mutex_unlock(&pipe->busy_mutex);
               return 1;
             }
             dt_opencl_release_mem_object(cl_mem_input);
+            valid_input_on_gpu_only = FALSE;
           }
 
-          /* process module on cpu */
-          module->process(module, piece, input, *output, &roi_in, roi_out);
+          /* process module on cpu. use tiling if needed and possible. */
+          if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
+                !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
+                max(in_bpp, bpp), tiling.factor, tiling.overhead))
+            module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
+          else
+            module->process(module, piece, input, *output, &roi_in, roi_out);
+
           /* process blending on cpu */
           dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
         }
@@ -596,34 +664,56 @@ dt_dev_pixelpipe_process_rec(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, void *
           // if (rand() % 5 == 0) err = !CL_SUCCESS; // Test code: simulate spurious failures
           if (err != CL_SUCCESS)
           {
-            /* fatal opencl error */
-            dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (b)] fatal opencl error while copying back to cpu buffer: %d\n", err);
+            /* late opencl error */
+            dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (c)] late opencl error detected while copying back to cpu buffer: %d\n", err);
             dt_opencl_release_mem_object(cl_mem_input);
             pipe->opencl_error = 1;
             dt_pthread_mutex_unlock(&pipe->busy_mutex);
             return 1;
           }
           dt_opencl_release_mem_object(cl_mem_input);
+          valid_input_on_gpu_only = FALSE;
         }
 
-        /* process module on cpu */
-        module->process(module, piece, input, *output, &roi_in, roi_out);
+        /* process module on cpu. use tiling if needed and possible. */
+        if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
+              !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
+              max(in_bpp, bpp), tiling.factor, tiling.overhead))
+          module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
+        else
+          module->process(module, piece, input, *output, &roi_in, roi_out);
+
         /* process blending */
         dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
       }
+
+      /* input is still only on GPU? Let's invalidate CPU input buffer then */
+      if (valid_input_on_gpu_only) dt_dev_pixelpipe_cache_invalidate(&(pipe->cache), input);
     }
     else
     {
-      /* opencl is not inited, everything runs on cpu */
+      /* opencl is not inited or not enabled or we got no resource/device -> everything runs on cpu */
+
+      /* process module on cpu. use tiling if needed and possible. */
+      if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
+            !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
+            max(in_bpp, bpp), tiling.factor, tiling.overhead))
+        module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
+      else
+        module->process(module, piece, input, *output, &roi_in, roi_out);
 
-      /* process module on cpu */
-      module->process(module, piece, input, *output, &roi_in, roi_out);
       /* process blending */
       dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
     } 
 #else
-    /* process module on cpu */
-    module->process(module, piece, input, *output, &roi_in, roi_out);
+    /* process module on cpu. use tiling if needed and possible. */
+    if((module->flags() & IOP_FLAGS_ALLOW_TILING) && 
+          !dt_tiling_piece_fits_host_memory(max(roi_in.width, roi_out->width), max(roi_in.height, roi_out->height), 
+          max(in_bpp, bpp), tiling.factor, tiling.overhead))
+      module->process_tiling(module, piece, input, *output, &roi_in, roi_out, in_bpp);
+    else
+      module->process(module, piece, input, *output, &roi_in, roi_out);
+
     /* process blending */
     dt_develop_blend_process(module, piece, input, *output, &roi_in, roi_out);
 #endif
@@ -782,8 +872,8 @@ dt_dev_pixelpipe_process_rec_and_backcopy(dt_dev_pixelpipe_t *pipe, dt_develop_t
 
       if (err != CL_SUCCESS)
       {
-        /* this indicates a serious opencl problem */
-        dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (c)] fatal opencl error while copying back to cpu buffer: %d\n", err);
+        /* this indicates a opencl problem earlier in the pipeline */
+        dt_print(DT_DEBUG_OPENCL, "[opencl_pixelpipe (d)] late opencl error detected while copying back to cpu buffer: %d\n", err);
         pipe->opencl_error = 1;
         ret = 1;
       }
@@ -802,9 +892,12 @@ int dt_dev_pixelpipe_process(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, int x,
 {
   pipe->processing = 1;
   pipe->opencl_enabled = dt_opencl_update_enabled(); // update enabled flag from preferences
-  pipe->devid = dt_opencl_lock_device(-1);
+  pipe->devid = pipe->opencl_enabled ? dt_opencl_lock_device(-1) : -1;  // try to get/lock opencl resource
+
   dt_print(DT_DEBUG_OPENCL, "[pixelpipe_process] [%s] using device %d\n", pipe->type == DT_DEV_PIXELPIPE_PREVIEW ? "preview" : (pipe->type == DT_DEV_PIXELPIPE_FULL ? "full" : "export"), pipe->devid);
-  dt_opencl_events_reset(pipe->devid);
+
+  if(pipe->devid >= 0) dt_opencl_events_reset(pipe->devid);
+
   dt_iop_roi_t roi = (dt_iop_roi_t)
   {
     x, y, width, height, scale
@@ -818,38 +911,47 @@ int dt_dev_pixelpipe_process(dt_dev_pixelpipe_t *pipe, dt_develop_t *dev, int x,
   GList *modules = g_list_last(dev->iop);
   GList *pieces = g_list_last(pipe->nodes);
 
-  // re-entry point: in case of fatal OpenCL errors we start all over again with OpenCL disabled
+  // re-entry point: in case of late opencl errors we start all over again with opencl-support disabled
 restart:
+
   // image max is normalized before
   for(int k=0; k<3; k++) pipe->processed_maximum[k] = 1.0f; // dev->image->maximum;
+
   void *buf = NULL;
   void *cl_mem_out = NULL;
   int out_bpp;
+  int oclerr = 0;
+
   // run pixelpipe recursively and get error status
   int err = dt_dev_pixelpipe_process_rec_and_backcopy(pipe, dev, &buf, &cl_mem_out, &out_bpp, &roi, modules, pieces, pos);
-  // check error status of OpenCL queue
-  int oclerr = (dt_opencl_events_flush(pipe->devid, 0) != CL_COMPLETE);
 
-  // OpenCL errors can come in two ways: pipe->opencl_error is TRUE or oclerr is TRUE
-  // if we have OpenCL errors ....
+  // get status summary of opencl queue by checking the eventlist
+  if(pipe->devid >= 0) oclerr = (dt_opencl_events_flush(pipe->devid, 1) != 0);
+
+  // Check if we had opencl errors ....
+  // remark: opencl errors can come in two ways: pipe->opencl_error is TRUE (and err is TRUE) OR oclerr is TRUE
   if (oclerr || (err && pipe->opencl_error))
   {
-    // we might need to free an invalid OpenCL memory object
+    // Well, there were error -> we might need to free an invalid opencl memory object
     if (cl_mem_out != NULL) dt_opencl_release_mem_object(cl_mem_out);
-    // dt_opencl_disable();
-    // dt_control_log("Warning: OpenCL was found to be unreliable on this system and is therefore disabled!");
+    dt_opencl_unlock_device(pipe->devid); // release opencl resource
     dt_pthread_mutex_lock(&pipe->busy_mutex);
-    pipe->opencl_enabled = 0; // disable OpenCL for this pipe
-    pipe->opencl_error = 0;
+    pipe->opencl_enabled = 0;             // disable opencl for this pipe
+    pipe->opencl_error = 0;               // reset error status
+    pipe->devid = -1;
+    dt_pthread_mutex_unlock(&pipe->busy_mutex);
     dt_dev_pixelpipe_flush_caches(pipe);
     dt_dev_pixelpipe_change(pipe, dev);
-    dt_pthread_mutex_unlock(&pipe->busy_mutex);
-    goto restart;  // (as said before)
+    dt_print(DT_DEBUG_OPENCL, "[pixelpipe_process] [%s] falling back to cpu path\n", pipe->type == DT_DEV_PIXELPIPE_PREVIEW ? "preview" : (pipe->type == DT_DEV_PIXELPIPE_FULL ? "full" : "export"));
+    goto restart;  // try again (this time without opencl)
   }
 
   // release resources:
-  dt_opencl_unlock_device(pipe->devid);
-  pipe->devid = -1;
+  if(pipe->devid >= 0)
+  {
+    dt_opencl_unlock_device(pipe->devid);
+    pipe->devid = -1;
+  }
   // ... and in case of other errors ...
   if (err)
   {
