@@ -921,7 +921,7 @@ store (dt_imageio_module_data_t *sdata, const int imgid, dt_imageio_module_forma
 
   /* construct a temporary file name */
   char fname[4096]= {0};
-  dt_get_user_local_dir (fname,4096);
+  dt_util_get_user_local_dir (fname,4096);
   g_strlcat (fname,"/tmp",4096);
   g_mkdir_with_parents(fname,0700);
   g_strlcat (fname,"/darktable.XXXXXX.",4096);
