@@ -92,7 +92,7 @@ int flags()
   return IOP_FLAGS_DEPRECATED;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
 #if 0 // we are deprecated.
   dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/anlfyeni/sensitivity");
