@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    copyright (c) 2009--2010 johannes hanika.
+    copyright (c) 2009--2011 johannes hanika.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,9 +24,8 @@
 #include <gtk/gtk.h>
 #include <inttypes.h>
 #include <gdk/gdkkeysyms.h>
-#ifdef HAVE_GEGL
-#include <gegl.h>
-#endif
+#include <assert.h>
+
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "control/control.h"
@@ -37,12 +36,16 @@
 #include "dtgtk/resetlabel.h"
 #include "dtgtk/togglebutton.h"
 #include "dtgtk/button.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "gui/draw.h"
 #include "gui/presets.h"
 
 DT_MODULE(3)
 
+// number of gui ratios in combo box
+#define NUM_RATIOS 9
+
 /** flip H/V, rotate an image, then clip the buffer. */
 typedef enum dt_iop_clipping_flags_t
 {
@@ -89,20 +92,19 @@ legacy_params (dt_iop_module_t *self, const void *const old_params, const int ol
 typedef struct dt_iop_clipping_gui_data_t
 {
   GtkDarktableSlider *scale5, *keystone_h,*keystone_v;
+  GtkWidget *swap_button;
   GtkDarktableToggleButton *hflip,*vflip;
   GtkComboBoxEntry *aspect_presets;
   GtkComboBox *guide_lines;
   GtkLabel *label7;
   GtkDarktableToggleButton *flipHorGoldenGuide, *flipVerGoldenGuide;
   GtkCheckButton *goldenSectionBox, *goldenSpiralSectionBox, *goldenSpiralBox, *goldenTriangleBox;
-  GClosure *commit_callback;
-  GClosure *undo_callback;
 
   float button_down_zoom_x, button_down_zoom_y, button_down_angle; // position in image where the button has been pressed.
   float clip_x, clip_y, clip_w, clip_h, handle_x, handle_y;
   float old_clip_x, old_clip_y, old_clip_w, old_clip_h;
-  int cropping, straightening;
-  float aspect_ratios[9];
+  int cropping, straightening, applied;
+  float aspect_ratios[NUM_RATIOS];
   float current_aspect;
 }
 dt_iop_clipping_gui_data_t;
@@ -163,6 +165,20 @@ operation_tags ()
   return IOP_TAG_DISTORT;
 }
 
+int
+operation_tags_filter ()
+{
+  // switch off watermark, it gets confused.
+  return IOP_TAG_DECORATION;
+}
+
+
+static int
+gui_has_focus(struct dt_iop_module_t *self)
+{
+  return self->dev->gui_module == self;
+}
+
 static void
 backtransform(float *x, float *o, const float *m, const float t_h, const float t_v)
 {
@@ -407,10 +423,6 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)p1;
-#ifdef HAVE_GEGL
-  // pull in new params to gegl
-#error "clipping needs to be ported to GEGL!"
-#else
   dt_iop_clipping_data_t *d = (dt_iop_clipping_data_t *)piece->data;
   // pull in bit from weird p->k => d->keystone = 1
   d->all_off = 1;
@@ -421,39 +433,71 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pi
   if(p->k_v >= -1.0 && p->k_v <= 1.0) d->ki_v = p->k_v;
   else d->ki_v = 0.0f;
   d->angle = M_PI/180.0 * p->angle;
-  d->cx = p->cx;
-  d->cy = p->cy;
-  d->cw = fabsf(p->cw);
-  d->ch = fabsf(p->ch);
+  if(gui_has_focus(self))
+  {
+    d->cx = 0.0f;
+    d->cy = 0.0f;
+    d->cw = 1.0f;
+    d->ch = 1.0f;
+  }
+  else
+  {
+    d->cx = p->cx;
+    d->cy = p->cy;
+    d->cw = fabsf(p->cw);
+    d->ch = fabsf(p->ch);
+  }
   d->flags = (p->ch < 0 ? FLAG_FLIP_VERTICAL : 0) | (p->cw < 0 ? FLAG_FLIP_HORIZONTAL : 0);
-#endif
+}
+
+void gui_focus (struct dt_iop_module_t *self, gboolean in)
+{
+  dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
+  dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)self->params;
+  if(self->enabled)
+  {
+    if(in)
+    {
+      // got focus. make it redraw in full and grab stuff to gui:
+      // need to get gui stuff for the first time for this image,
+      // and advice the pipe to redraw in full:
+      g->clip_x = p->cx;
+      g->clip_w = p->cw - p->cx;
+      g->clip_y = p->cy;
+      g->clip_h = p->ch - p->cy;
+      // flip one bit to trigger the cache:
+      uint32_t hack = *(uint32_t*)&p->cy;
+      hack ++;
+      p->cy = *(float *)&hack;
+      if(!darktable.gui->reset)
+        dt_dev_add_history_item(darktable.develop, self, TRUE);
+    }
+    else
+    {
+      // lost focus, commit current params:
+      commit_box (self, g, p);
+    }
+  }
 }
 
 void init_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
-#ifdef HAVE_GEGL
-#error "clipping needs to be ported to GEGL!"
-#else
   piece->data = malloc(sizeof(dt_iop_clipping_data_t));
   self->commit_params(self, self->default_params, pipe, piece);
-#endif
 }
 
 void cleanup_pipe (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
-#ifdef HAVE_GEGL
-#error "clipping needs to be ported to GEGL!"
-#else
   free(piece->data);
-#endif
 }
 
 static void
 apply_box_aspect(dt_iop_module_t *self, int grab)
 {
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
-  float wd = self->dev->preview_pipe->backbuf_width;
-  float ht = self->dev->preview_pipe->backbuf_height;
+  int iwd, iht;
+  dt_dev_get_processed_size(darktable.develop, &iwd, &iht);
+  float wd = iwd, ht = iht;
   // enforce aspect ratio.
   const float aspect = g->current_aspect;
   // const float aspect = gtk_spin_button_get_value(g->aspect);
@@ -539,7 +583,7 @@ apply_box_aspect(dt_iop_module_t *self, int grab)
   }
 }
 
-void init_presets (dt_iop_module_t *self)
+void init_presets (dt_iop_module_so_t *self)
 {
   dt_iop_clipping_params_t p = (dt_iop_clipping_params_t)
   {
@@ -547,11 +591,11 @@ void init_presets (dt_iop_module_t *self)
   };
   DT_DEBUG_SQLITE3_EXEC(darktable.db, "begin", NULL, NULL, NULL);
   p.angle = 90.0f;
-  dt_gui_presets_add_generic(_("rotate by  90"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("rotate by  90"), self->op, self->version(), &p, sizeof(p), 1);
   p.angle = -90.0f;
-  dt_gui_presets_add_generic(_("rotate by -90"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("rotate by -90"), self->op, self->version(), &p, sizeof(p), 1);
   p.angle = 180.0f;
-  dt_gui_presets_add_generic(_("rotate by 180"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("rotate by 180"), self->op, self->version(), &p, sizeof(p), 1);
   DT_DEBUG_SQLITE3_EXEC(darktable.db, "commit", NULL, NULL, NULL);
 }
 
@@ -575,6 +619,28 @@ void reload_defaults(dt_iop_module_t *self)
   memcpy(self->default_params, &tmp, sizeof(dt_iop_clipping_params_t));
 }
 
+static void
+aspect_free_activated (GtkEntry *entry, dt_iop_module_t *self)
+{
+  dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
+  gchar *text = g_strdup(gtk_entry_get_text(entry));
+  if(text)
+  {
+    gchar *c = text;
+    while(*c != ':' && *c != '/' && c < text + strlen(text)) c++;
+    if(c < text + strlen(text) - 1)
+    {
+      *c = '\0';
+      c++;
+      g->current_aspect = atof(text) / atof(c);
+      apply_box_aspect(self, 5);
+      dt_control_queue_draw_all();
+      dt_iop_request_focus(self);
+    }
+    g_free(text);
+  }
+}
+
 static void
 aspect_presets_changed (GtkComboBox *combo, dt_iop_module_t *self)
 {
@@ -582,27 +648,23 @@ aspect_presets_changed (GtkComboBox *combo, dt_iop_module_t *self)
   int which = gtk_combo_box_get_active(combo);
   if (which < 0)
   {
-    // reset to free aspect ratio:
-    g->current_aspect = -1.0;
-    dt_conf_set_int("plugins/darkroom/clipping/aspect_preset", -1);
-    gchar *text = gtk_combo_box_get_active_text(combo);
-    if(text)
+    // parse config param:
+    if(g->current_aspect == -1.0f)
     {
-      gchar *c = text;
-      while(*c != ':' && *c != '/' && c < text + strlen(text)) c++;
-      if(c < text + strlen(text) - 1)
-      {
-        *c = '\0';
-        c++;
-        g->current_aspect = atof(text) / atof(c);
-        apply_box_aspect(self, 5);
-        dt_control_queue_draw_all();
-        dt_iop_request_focus(self);
-      }
-      g_free(text);
+      g->current_aspect = dt_conf_get_float("plugins/darkroom/clipping/custom_aspect");
+      if(g->current_aspect <= 0.0f) g->current_aspect = 1.5f;
+      char text[128];
+      snprintf(text, 128, "%.3f:1", g->current_aspect);
+      gtk_entry_set_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(combo))), text);
+      apply_box_aspect(self, 5);
+      dt_control_queue_draw_all();
     }
+    // user is typing, don't overwrite it.
+    g->current_aspect = -2.0f;
+    // reset to free aspect ratio:
+    dt_conf_set_int("plugins/darkroom/clipping/aspect_preset", -1);
   }
-  else if (which < 9)
+  else if (which < NUM_RATIOS)
   {
     dt_conf_set_int("plugins/darkroom/clipping/aspect_preset", which);
     if(which > 1 && self->dev->image->height > self->dev->image->width)
@@ -619,9 +681,10 @@ static void
 angle_callback (GtkDarktableSlider *slider, dt_iop_module_t *self)
 {
   if(self->dt->gui->reset) return;
+  dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
   dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)self->params;
   p->angle = dtgtk_slider_get_value(slider);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  commit_box (self, g, p);
 }
 
 static void
@@ -632,7 +695,7 @@ keystone_callback_h (GtkWidget *widget, dt_iop_module_t *self)
   dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)self->params;
   // we need k to be abs(k) < 2, so the second bit will always be zero (except we set it:).
   p->k_h = fmaxf(-1.9, fminf(1.9, dtgtk_slider_get_value(g->keystone_h)));
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  commit_box (self, g, p);
 }
 static void
 keystone_callback_v (GtkWidget *widget, dt_iop_module_t *self)
@@ -642,7 +705,7 @@ keystone_callback_v (GtkWidget *widget, dt_iop_module_t *self)
   dt_iop_clipping_params_t *p = (dt_iop_clipping_params_t *)self->params;
   // we need k to be abs(k) < 2, so the second bit will always be zero (except we set it:).
   p->k_v = fmaxf(-1.9, fminf(1.9, dtgtk_slider_get_value(g->keystone_v)));
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  commit_box (self, g, p);
 }
 
 void gui_update(struct dt_iop_module_t *self)
@@ -655,8 +718,15 @@ void gui_update(struct dt_iop_module_t *self)
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->hflip), p->cw < 0);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->vflip), p->ch < 0);
   int act = dt_conf_get_int("plugins/darkroom/clipping/aspect_preset");
-  if(act < 0 || act > 7) act = 0;
+  if(act < -1 || act >= NUM_RATIOS) act = 0;
   gtk_combo_box_set_active(GTK_COMBO_BOX(g->aspect_presets), act);
+
+  // reset gui draw box to what we have in the parameters:
+  g->applied = 1;
+  g->clip_x = p->cx;
+  g->clip_w = p->cw - p->cx;
+  g->clip_y = p->cy;
+  g->clip_h = p->ch - p->cy;
 }
 
 void init(dt_iop_module_t *module)
@@ -695,7 +765,7 @@ toggled_callback(GtkDarktableToggleButton *widget, dt_iop_module_t *self)
     else                                     p->ch = copysignf(p->ch,  1.0);
   }
   if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
+  commit_box (self, g, p);
 }
 
 static void
@@ -724,26 +794,6 @@ static void key_commit_callback(GtkAccelGroup *accel_group,
   commit_box(self, g, p);
 }
 
-static void key_undo_callback(GtkAccelGroup *accel_group,
-                              GObject *acceleratable,
-                              guint keyval, GdkModifierType modifier,
-                              gpointer data)
-{
-  dt_iop_module_t* self = (dt_iop_module_t*)data;
-  dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
-  dt_iop_clipping_params_t   *p = (dt_iop_clipping_params_t   *)self->params;
-
-  // reverse cropping to where it was before.
-  p->cx = p->cy = 0.0f;
-  p->cw = p->ch = 1.0f;
-  g->clip_x = g->old_clip_x;
-  g->clip_y = g->old_clip_y;
-  g->clip_w = g->old_clip_w;
-  g->clip_h = g->old_clip_h;
-  dt_dev_add_history_item(darktable.develop, self, TRUE);
-  dt_control_queue_draw_all();
-}
-
 static void
 aspect_flip(GtkWidget *button, dt_iop_module_t *self)
 {
@@ -819,6 +869,7 @@ void gui_init(struct dt_iop_module_t *self)
   g->old_clip_w = g->old_clip_h = 1.0;
   g->cropping = 0;
   g->straightening = 0;
+  g->applied = 1;
 
   self->widget = gtk_table_new(10, 6, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(self->widget), DT_GUI_IOP_MODULE_CONTROL_SPACING);
@@ -844,7 +895,6 @@ void gui_init(struct dt_iop_module_t *self)
   g_signal_connect (G_OBJECT (g->scale5), "value-changed",
                     G_CALLBACK (angle_callback), self);
   g_object_set(G_OBJECT(g->scale5), "tooltip-text", _("right-click and drag a line on the image to drag a straight line"), (char *)NULL);
-  dtgtk_slider_set_accel(g->scale5,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/angle");
   gtk_table_attach(GTK_TABLE(self->widget), GTK_WIDGET(g->scale5), 0, 6, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
 
@@ -853,7 +903,6 @@ void gui_init(struct dt_iop_module_t *self)
   g_object_set(G_OBJECT(g->keystone_h), "tooltip-text", _("adjust perspective for horizontal keystone distortion"), (char *)NULL);
   g_signal_connect (G_OBJECT (g->keystone_h), "value-changed",
                     G_CALLBACK (keystone_callback_h), self);
-  dtgtk_slider_set_accel(g->keystone_h,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/keystone h");
   gtk_table_attach(GTK_TABLE(self->widget), GTK_WIDGET(g->keystone_h), 0, 6, 2, 3, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
   g->keystone_v = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, -1.0, 1.0, 0.01, 0.0, 2));
@@ -861,7 +910,6 @@ void gui_init(struct dt_iop_module_t *self)
   g_object_set(G_OBJECT(g->keystone_v), "tooltip-text", _("adjust perspective for vertical keystone distortion"), (char *)NULL);
   g_signal_connect (G_OBJECT (g->keystone_v), "value-changed",
                     G_CALLBACK (keystone_callback_v), self);
-  dtgtk_slider_set_accel(g->keystone_v,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/keystone v");
   gtk_table_attach(GTK_TABLE(self->widget), GTK_WIDGET(g->keystone_v), 0, 6, 3, 4, GTK_EXPAND|GTK_FILL, 0, 0, 0);
 
   label = gtk_label_new(_("aspect"));
@@ -878,27 +926,20 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("square"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("DIN"));
   gtk_combo_box_append_text(GTK_COMBO_BOX(g->aspect_presets), _("16:9"));
-  g->commit_callback = g_cclosure_new(G_CALLBACK(key_commit_callback),
-                                      (gpointer)self, NULL);
-  g->undo_callback = g_cclosure_new(G_CALLBACK(key_undo_callback),
-                                    (gpointer)self, NULL);
-  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
-                                 "<Darktable>/darkroom/plugins/clipping/commit",
-                                 g->commit_callback);
-  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
-                                 "<Darktable>/darkroom/plugins/clipping/undo",
-                                 g->undo_callback);
+  dt_gui_key_accel_block_on_focus(gtk_bin_get_child(GTK_BIN(g->aspect_presets)));
   int act = dt_conf_get_int("plugins/darkroom/clipping/aspect_preset");
   if(act < 0 || act >= 9) act = 0;
   gtk_combo_box_set_active(GTK_COMBO_BOX(g->aspect_presets), act);
   g_signal_connect (G_OBJECT (g->aspect_presets), "changed",
                     G_CALLBACK (aspect_presets_changed), self);
+  g_signal_connect (G_OBJECT (gtk_bin_get_child(GTK_BIN(g->aspect_presets))), "activate",
+                    G_CALLBACK (aspect_free_activated), self);
   g_object_set(G_OBJECT(g->aspect_presets), "tooltip-text", _("set the aspect ratio (w:h)\npress ctrl-x to swap sides"), (char *)NULL);
 
   GtkBox *hbox = GTK_BOX(gtk_hbox_new(FALSE, 5));
   gtk_box_pack_start(hbox, GTK_WIDGET(g->aspect_presets), TRUE, TRUE, 0);
   GtkWidget *button = dtgtk_button_new(dtgtk_cairo_paint_aspectflip, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/swap the aspect ratio");
+  g->swap_button = GTK_WIDGET(button);
   g_signal_connect (G_OBJECT (button), "clicked", G_CALLBACK (aspect_flip), self);
   g_object_set(G_OBJECT(button), "tooltip-text", _("swap the aspect ratio (ctrl-x)"), (char *)NULL);
   gtk_box_pack_start(hbox, button, TRUE, FALSE, 0);
@@ -990,6 +1031,8 @@ void gui_init(struct dt_iop_module_t *self)
   g->aspect_ratios[6] = 1.0;
   g->aspect_ratios[7] = sqrtf(2.0);
   g->aspect_ratios[8] = 16.0f/9.0f;
+  // if adding new presets, make sure to change this as well:
+  assert(NUM_RATIOS == 9);
 
   if(act> 0 && self->dev->image->height > self->dev->image->width)
     g->current_aspect = 1.0/g->aspect_ratios[act];
@@ -999,12 +1042,6 @@ void gui_init(struct dt_iop_module_t *self)
 
 void gui_cleanup(struct dt_iop_module_t *self)
 {
-  dt_accel_group_disconnect(darktable.control->accels_darkroom,
-                             ((dt_iop_clipping_gui_data_t*)(self->gui_data))->
-                             commit_callback);
-  dt_accel_group_disconnect(darktable.control->accels_darkroom,
-                             ((dt_iop_clipping_gui_data_t*)(self->gui_data))->
-                             undo_callback);
   free(self->gui_data);
   self->gui_data = NULL;
 }
@@ -1198,6 +1235,7 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
 {
   dt_develop_t *dev = self->dev;
   dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t *)self->gui_data;
+
   int32_t zoom, closeup;
   float zoom_x, zoom_y;
   float wd = dev->preview_pipe->backbuf_width;
@@ -1220,7 +1258,10 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   pzx += 0.5f;
   pzy += 0.5f;
   cairo_set_dash (cr, &dashes, 0, 0);
-  cairo_set_source_rgba(cr, .3, .3, .3, .8);
+  if(g->applied)
+    cairo_set_source_rgba(cr, .2, .2, .2, .8);
+  else
+    cairo_set_source_rgba(cr, .3, .3, .3, .5);
   cairo_set_fill_rule(cr, CAIRO_FILL_RULE_EVEN_ODD);
   cairo_rectangle (cr, -1, -1, wd+2, ht+2);
   cairo_rectangle (cr, g->clip_x*wd, g->clip_y*ht, g->clip_w*wd, g->clip_h*ht);
@@ -1388,6 +1429,8 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, int which)
   }
   else if(darktable.control->button_down && darktable.control->button_down_which == 1)
   {
+    // draw a light gray frame, to show it's not stored yet:
+    g->applied = 0;
     // first mouse button, adjust cropping frame, but what do we do?
     float bzx = g->button_down_zoom_x + .5f, bzy = g->button_down_zoom_y + .5f;
     if(!g->cropping && !g->straightening)
@@ -1470,6 +1513,7 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, int which)
 static void
 commit_box (dt_iop_module_t *self, dt_iop_clipping_gui_data_t *g, dt_iop_clipping_params_t *p)
 {
+  if(darktable.gui->reset) return;
   g->old_clip_x = g->clip_x;
   g->old_clip_y = g->clip_y;
   g->old_clip_w = g->clip_w;
@@ -1481,21 +1525,13 @@ commit_box (dt_iop_module_t *self, dt_iop_clipping_gui_data_t *g, dt_iop_clippin
     p->cx = p->cy = 0.0f;
     p->cw = p->ch = 1.0f;
   }
-  const float cx = p->cx, cy = p->cy;
-  const float cw = fabsf(p->cw), ch = fabsf(p->ch);
-  p->cx += g->clip_x*(cw-cx);
-  p->cy += g->clip_y*(ch-cy);
-  p->cw = copysignf(p->cx + (cw - cx)*g->clip_w, p->cw);
-  p->ch = copysignf(p->cy + (ch - cy)*g->clip_h, p->ch);
-  g->clip_x = g->clip_y = 0.0f;
-  g->clip_w = g->clip_h = 1.0;
-  darktable.gui->reset = 1;
-  self->gui_update(self);
-  darktable.gui->reset = 0;
+  p->cx = g->clip_x;
+  p->cy = g->clip_y;
+  p->cw = copysignf(p->cx + g->clip_w, p->cw);
+  p->ch = copysignf(p->cy + g->clip_h, p->ch);
   if(self->off) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(self->off), 1);
+  g->applied = 1;
   dt_dev_add_history_item(darktable.develop, self, TRUE);
-  // loose focus, continue with other plugins?
-  darktable.develop->gui_module = NULL;
 }
 
 int button_released(struct dt_iop_module_t *self, double x, double y, int which, uint32_t state)
@@ -1544,24 +1580,30 @@ int button_pressed(struct dt_iop_module_t *self, double x, double y, int which,
   else return 0;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  gtk_accel_map_add_entry("<Darktable>/darkroom/plugins/clipping/commit",
-                          GDK_Return, 0);
-  gtk_accel_map_add_entry("<Darktable>/darkroom/plugins/clipping/undo",
-                          GDK_z, GDK_CONTROL_MASK);
+  dt_accel_register_iop(self, TRUE, NC_("accel", "commit"),
+                        GDK_Return, 0);
+  dt_accel_register_iop(self, TRUE, NC_("accel", "swap the aspect ratio"),
+                        GDK_x, 0);
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "angle"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "keystone h"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "keystone v"));
+}
 
-  // Making sure these get into the accelerator lists as well
-  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
-                                 "<Darktable>/darkroom/plugins/clipping/commit",
-                                 NULL);
-  dt_accel_group_connect_by_path(darktable.control->accels_darkroom,
-                                 "<Darktable>/darkroom/plugins/clipping/undo",
-                                 NULL);
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/angle");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/keystone h");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/keystone v");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/clipping/swap the aspect ratio");
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_clipping_gui_data_t *g = (dt_iop_clipping_gui_data_t*)self->gui_data;
+  GClosure *closure;
+
+  closure = g_cclosure_new(G_CALLBACK(key_commit_callback),
+                           (gpointer)self, NULL);
+  dt_accel_connect_iop(self, "commit", closure);
+
+  dt_accel_connect_button_iop(self, "swap the aspect ratio", g->swap_button);
+  dt_accel_connect_slider_iop(self, "angle", GTK_WIDGET(g->scale5));
+  dt_accel_connect_slider_iop(self, "keystone h", GTK_WIDGET(g->keystone_h));
+  dt_accel_connect_slider_iop(self, "keystone v", GTK_WIDGET(g->keystone_v));
 }
 
 #undef PHI
@@ -1573,4 +1615,6 @@ void init_key_accels()
 #undef GUIDE_TRIANGL
 #undef GUIDE_GOLDEN
 
+#undef NUM_RATIOS
+
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-space on;
