@@ -23,10 +23,12 @@
 #include "develop/imageop.h"
 #include "control/control.h"
 #include "dtgtk/slider.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include <gtk/gtk.h>
 #include <stdlib.h>
 #include <assert.h>
+#include <xmmintrin.h>
 
 DT_MODULE(1)
 
@@ -98,10 +100,21 @@ groups ()
   return IOP_GROUP_COLOR;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/colorcontrast/green vs magenta");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/colorcontrast/blue vs yellow");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "green vs magenta"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blue vs yellow"));
+}
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_colorcontrast_gui_data_t *g =
+      (dt_iop_colorcontrast_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_slider_iop(self, "green vs magenta",
+                              GTK_WIDGET(g->a_scale));
+  dt_accel_connect_slider_iop(self, "blue vs yellow",
+                              GTK_WIDGET(g->b_scale));
 }
 
 /** modify regions of interest (optional, per pixel ops don't need this) */
@@ -124,20 +137,24 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 #endif
   for(int j=0; j<roi_out->height; j++)
   {
+
     float *in  = ((float *)i) + ch*roi_in->width *j;
     float *out = ((float *)o) + ch*roi_out->width*j;
+
+    const __m128 scale = _mm_set_ps(0.0f,d->b_steepness,d->a_steepness,1.0f);
+    const __m128 offset = _mm_set_ps(0.0f,d->b_offset,d->a_offset,0.0f);
+    const __m128 min = _mm_set_ps(0.0f,-128.0f,-128.0f, -INFINITY);
+    const __m128 max = _mm_set_ps(0.0f, 128.0f, 128.0f,  INFINITY);
+
+
     for(int i=0; i<roi_out->width; i++)
     {
-      float *pt_in = in + ch*i;
-      float *pt_out = out + ch*i;
-      float L_in = pt_in[0];
-      float a_in = pt_in[1];
-      float b_in = pt_in[2];
-      pt_out[0] = L_in;
-      pt_out[1] = CLAMPS(a_in*(d->a_steepness) + (d->a_offset), -128.0, 128.0);
-      pt_out[2] = CLAMPS(b_in*(d->b_steepness) + (d->b_offset), -128.0, 128.0);
+      _mm_stream_ps(out,_mm_min_ps(max,_mm_max_ps(min,_mm_add_ps(offset,_mm_mul_ps(scale,_mm_load_ps(in))))));
+      in+=ch;
+      out+=ch;
     }
   }
+  _mm_sfence();
 }
 
 /** optional: if this exists, it will be called to init new defaults if a new image is loaded from film strip mode. */
@@ -194,7 +211,7 @@ void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *params, dt_de
   d->a_steepness = p->a_steepness;
   d->a_offset = p->a_offset;
   d->b_steepness = p->b_steepness;
-  d->a_offset = p->a_offset;
+  d->b_offset = p->b_offset;
 }
 
 void init_pipe     (struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -251,10 +268,8 @@ void gui_init     (dt_iop_module_t *self)
   dt_iop_colorcontrast_params_t *p = (dt_iop_colorcontrast_params_t *)self->params;
   g->a_scale = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 5.0, 0.01, p->a_steepness, 2));
   dtgtk_slider_set_label(g->a_scale,_("green vs magenta"));
-  dtgtk_slider_set_accel(g->a_scale,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/colorcontrast/green vs magenta");
   g->b_scale = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 5.0, 0.01, p->b_steepness, 2));
   dtgtk_slider_set_label(g->b_scale,_("blue vs yellow"));
-  dtgtk_slider_set_accel(g->b_scale,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/colorcontrast/blue vs yellow");
   
   self->widget = GTK_WIDGET(gtk_hbox_new(FALSE, 0));
   g->vbox = GTK_VBOX(gtk_vbox_new(FALSE, DT_GUI_IOP_MODULE_CONTROL_SPACING));
