@@ -32,6 +32,7 @@
 #include "common/opencl.h"
 #include "dtgtk/resetlabel.h"
 #include "external/adobe_coeff.c"
+#include <xmmintrin.h>
 
 #define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
 
@@ -97,7 +98,7 @@ profile_changed (GtkComboBox *widget, gpointer user_data)
 {
   dt_iop_module_t *self = (dt_iop_module_t *)user_data;
   if(self->dt->gui->reset) return;
-  self->dev->gui_module = self;
+  dt_iop_request_focus(self);
   dt_iop_colorin_params_t *p = (dt_iop_colorin_params_t *)self->params;
   dt_iop_colorin_gui_data_t *g = (dt_iop_colorin_gui_data_t *)self->gui_data;
   int pos = gtk_combo_box_get_active(widget);
@@ -185,7 +186,36 @@ error:
 }
 #endif
 
+static inline __m128
+lab_f_m(const __m128 x)
+{
+  const __m128 epsilon = _mm_set1_ps(216.0f/24389.0f);
+  const __m128 kappa   = _mm_set1_ps(24389.0f/27.0f);
 
+  // calculate as if x > epsilon : result = cbrtf(x)
+  // approximate cbrtf(x):
+  const __m128 a = _mm_castsi128_ps(_mm_add_epi32(_mm_cvtps_epi32(_mm_div_ps(_mm_cvtepi32_ps(_mm_castps_si128(x)),_mm_set1_ps(3.0f))),_mm_set1_epi32(709921077)));
+  const __m128 a3 = _mm_mul_ps(_mm_mul_ps(a,a),a);
+  const __m128 res_big = _mm_div_ps(_mm_mul_ps(a,_mm_add_ps(a3,_mm_add_ps(x,x))),_mm_add_ps(_mm_add_ps(a3,a3),x));
+
+  // calculate as if x <= epsilon : result = (kappa*x+16)/116
+  const __m128 res_small = _mm_div_ps(_mm_add_ps(_mm_mul_ps(kappa,x),_mm_set1_ps(16.0f)),_mm_set1_ps(116.0f));
+
+  // blend results according to whether each component is > epsilon or not
+  const __m128 mask = _mm_cmpgt_ps(x,epsilon);
+  return _mm_or_ps(_mm_and_ps(mask,res_big),_mm_andnot_ps(mask,res_small));
+}
+
+static inline __m128
+dt_XYZ_to_Lab_SSE(const __m128 XYZ)
+{
+  const __m128 d50_inv  = _mm_set_ps(0.0f, 1.0f/0.8249f, 1.0f, 1.0f/0.9642f);
+  const __m128 coef = _mm_set_ps(0.0f,200.0f,500.0f,116.0f);
+  const __m128 f = lab_f_m(_mm_mul_ps(XYZ,d50_inv));
+  // because d50_inv.z is 0.0f, lab_f(0) == 16/116, so Lab[0] = 116*f[0] - 16 equal to 116*(f[0]-f[3])
+  return _mm_mul_ps(coef,_mm_sub_ps(_mm_shuffle_ps(f,f,_MM_SHUFFLE(3,1,0,1)),_mm_shuffle_ps(f,f,_MM_SHUFFLE(3,2,1,3))));
+}
+ 
 void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, void *o, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
   const dt_iop_colorin_data_t *const d = (dt_iop_colorin_data_t *)piece->data;
@@ -199,49 +229,59 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
   {
     // only color matrix. use our optimized fast path!
 #ifdef _OPENMP
-    #pragma omp parallel for default(none) shared(roi_out, out, in) schedule(static)
+    #pragma omp parallel for default(none) shared(roi_in,roi_out, out, in) schedule(static)
 #endif
-    for(int k=0; k<roi_out->width*roi_out->height; k++)
+    for(int j=0; j<roi_out->height; j++)
     {
-      const float *const buf_in  = in  + ch*k;
-      float *const buf_out = out + ch*k;
-      float cam[3], XYZ[3], Lab[3];
-      // memcpy(cam, buf_in, sizeof(float)*3);
-      // avoid calling this for linear profiles (marked with negative entries), assures unbounded
-      // color management without extrapolation.
-      for(int i=0; i<3; i++) cam[i] = (d->lut[i][0] >= 0.0f) ?
-        ((buf_in[i] < 1.0f) ? lerp_lut(d->lut[i], buf_in[i])
-        : dt_iop_eval_exp(d->unbounded_coeffs[i], buf_in[i]))
-        : buf_in[i];
 
-      if(map_blues)
+      float *buf_in  = in + ch*roi_in->width *j;
+      float *buf_out = out + ch*roi_out->width*j;
+      float cam[3];
+      const __m128 m0 = _mm_set_ps(0.0f,mat[6],mat[3],mat[0]);
+      const __m128 m1 = _mm_set_ps(0.0f,mat[7],mat[4],mat[1]);
+      const __m128 m2 = _mm_set_ps(0.0f,mat[8],mat[5],mat[2]);
+  
+      for(int i=0; i<roi_out->width; i++, buf_in+=ch, buf_out+=ch )
       {
-        // manual gamut mapping. these values cause trouble when converting back from Lab to sRGB.
-        // deeply saturated blues turn into purple fringes, so dampen them before conversion.
-        // this is off for non-raw images, which don't seem to have this problem.
-        // might be caused by too loose clipping bounds during highlight clipping?
-        const float YY = cam[0]+cam[1]+cam[2];
-        const float zz = cam[2]/YY;
-        // lower amount and higher bound_z make the effect smaller.
-        // the effect is weakened the darker input values are, saturating at bound_Y
-        const float bound_z = 0.5f, bound_Y = 0.8f;
-        const float amount = 0.11f;
-        if (zz > bound_z)
+  
+        // memcpy(cam, buf_in, sizeof(float)*3);
+        // avoid calling this for linear profiles (marked with negative entries), assures unbounded
+        // color management without extrapolation.
+        for(int i=0; i<3; i++) cam[i] = (d->lut[i][0] >= 0.0f) ?
+          ((buf_in[i] < 1.0f) ? lerp_lut(d->lut[i], buf_in[i])
+          : dt_iop_eval_exp(d->unbounded_coeffs[i], buf_in[i]))
+          : buf_in[i];
+  
+        if(map_blues)
         {
-          const float t = (zz - bound_z)/(1.0f-bound_z) * fminf(1.0, YY/bound_Y);
-          cam[1] += t*amount;
-          cam[2] -= t*amount;
+          // manual gamut mapping. these values cause trouble when converting back from Lab to sRGB.
+          // deeply saturated blues turn into purple fringes, so dampen them before conversion.
+          // this is off for non-raw images, which don't seem to have this problem.
+          // might be caused by too loose clipping bounds during highlight clipping?
+          const float YY = cam[0]+cam[1]+cam[2];
+          const float zz = cam[2]/YY;
+          // lower amount and higher bound_z make the effect smaller.
+          // the effect is weakened the darker input values are, saturating at bound_Y
+          const float bound_z = 0.5f, bound_Y = 0.8f;
+          const float amount = 0.11f;
+          if (zz > bound_z)
+          {
+            const float t = (zz - bound_z)/(1.0f-bound_z) * fminf(1.0f, YY/bound_Y);
+            cam[1] += t*amount;
+            cam[2] -= t*amount;
+          }
         }
+  
+#if 0
+        __attribute__((aligned(16))) float XYZ[4];
+        _mm_store_ps(XYZ,_mm_add_ps(_mm_add_ps( _mm_mul_ps(m0,_mm_set1_ps(cam[0])), _mm_mul_ps(m1,_mm_set1_ps(cam[1]))), _mm_mul_ps(m2,_mm_set1_ps(cam[2]))));
+        dt_XYZ_to_Lab(XYZ, buf_out);
+#endif
+        __m128 xyz = _mm_add_ps(_mm_add_ps( _mm_mul_ps(m0,_mm_set1_ps(cam[0])), _mm_mul_ps(m1,_mm_set1_ps(cam[1]))), _mm_mul_ps(m2,_mm_set1_ps(cam[2])));
+        _mm_stream_ps(buf_out,dt_XYZ_to_Lab_SSE(xyz));
       }
-      // now convert camera to XYZ using the color matrix
-      for(int j=0; j<3; j++)
-      {
-        XYZ[j] = 0.0f;
-        for(int i=0; i<3; i++) XYZ[j] += mat[3*j+i] * cam[i];
-      }
-      dt_XYZ_to_Lab(XYZ, Lab);
-      memcpy(buf_out, Lab, sizeof(float)*3);
     }
+    _mm_sfence();
   }
   else
   {
