@@ -3,7 +3,7 @@
 		copyright (c) 2009--2010 johannes hanika.
 
 		darktable is free software: you can redistribute it and/or modify
-		it under the terms of the GNU General Public License as published by
+                it under the terms of the GNU General Public License as published by
 		the Free Software Foundation, either version 3 of the License, or
 		(at your option) any later version.
 
@@ -28,6 +28,7 @@
 #include "develop/develop.h"
 #include "develop/imageop.h"
 #include "control/control.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "dtgtk/slider.h"
 #include "dtgtk/resetlabel.h"
@@ -93,11 +94,24 @@ flags ()
   return IOP_FLAGS_ALLOW_TILING;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend L");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend C");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend h");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blend L"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blend C"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blend h"));
+//  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend L");
+//  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend C");
+//  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend h");
+}
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_highlights_gui_data_t *g =
+      (dt_iop_highlights_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_slider_iop(self, "blend L", GTK_WIDGET(g->blendL));
+  dt_accel_connect_slider_iop(self, "blend C", GTK_WIDGET(g->blendC));
+  dt_accel_connect_slider_iop(self, "blend H", GTK_WIDGET(g->blendh));
 }
 
 static const float xyz_rgb[3][3] =    /* XYZ from RGB */
@@ -417,9 +431,6 @@ void gui_init(struct dt_iop_module_t *self)
   dtgtk_slider_set_label(g->blendL,_("blend L"));
   dtgtk_slider_set_label(g->blendC,_("blend C"));
   dtgtk_slider_set_label(g->blendh,_("blend h"));
-  dtgtk_slider_set_accel(g->blendL,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend L");
-  dtgtk_slider_set_accel(g->blendC,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend C");
-  dtgtk_slider_set_accel(g->blendh,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/highlights/blend h");
   dtgtk_slider_set_default_value(g->blendL, p->blendL);
   dtgtk_slider_set_default_value(g->blendC, p->blendC);
   dtgtk_slider_set_default_value(g->blendh, p->blendh);
