@@ -26,6 +26,7 @@
 #include "develop/imageop.h"
 #include "dtgtk/slider.h"
 #include "dtgtk/resetlabel.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include <gtk/gtk.h>
 #include <stdlib.h>
@@ -72,11 +73,21 @@ groups ()
   return IOP_GROUP_CORRECT;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/hotpixels/threshold");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/hotpixels/strength");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "threshold"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "strength"));
 }
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_hotpixels_gui_data_t *g =
+      (dt_iop_hotpixels_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_slider_iop(self, "threshold", GTK_WIDGET(g->threshold));
+  dt_accel_connect_slider_iop(self, "strength", GTK_WIDGET(g->strength));
+}
+
 int
 output_bpp(dt_iop_module_t *module, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
@@ -277,7 +288,6 @@ void gui_init     (dt_iop_module_t *self)
   dtgtk_slider_set_format_type(DTGTK_SLIDER(g->threshold),DARKTABLE_SLIDER_FORMAT_FLOAT);
   gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->threshold), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT (g->threshold), "value-changed", G_CALLBACK (threshold_callback), self);
-  dtgtk_slider_set_accel(g->threshold,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/hotpixels/threshold");
 
   g->strength = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0, 1.0, 0.01, p->strength, 4));
   dtgtk_slider_set_label(g->strength,_("strength"));
@@ -285,7 +295,6 @@ void gui_init     (dt_iop_module_t *self)
   dtgtk_slider_set_format_type(DTGTK_SLIDER(g->strength),DARKTABLE_SLIDER_FORMAT_FLOAT);
   gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->strength), TRUE, TRUE, 0);
   g_signal_connect(G_OBJECT (g->strength), "value-changed", G_CALLBACK (strength_callback), self);
-  dtgtk_slider_set_accel(g->strength,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/hotpixels/strength");
 
   g->permissive  = GTK_TOGGLE_BUTTON(gtk_check_button_new_with_label(_("detect by 3 neighbours")));
   gtk_toggle_button_set_active(g->permissive, p->permissive);
