@@ -33,6 +33,7 @@
 #include "control/control.h"
 #include "dtgtk/button.h"
 #include "dtgtk/resetlabel.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "gui/draw.h"
 #include "iop/lens.h"
@@ -57,20 +58,39 @@ operation_tags ()
   return IOP_TAG_DISTORT;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/scale");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/tca R");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/tca B");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/reset lens from exif data");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/reset camera from exif data");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/reset distance from exif data");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/find camera");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/find lens");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/auto scale");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/camera model");
-  dtgtk_button_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/lens model");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "scale"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "tca R"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "tca B"));
+
+  dt_accel_register_iop(self, FALSE, NC_("accel", "find camera"), 0, 0);
+  dt_accel_register_iop(self, FALSE, NC_("accel", "find lens"), 0, 0);
+  dt_accel_register_iop(self, FALSE, NC_("accel", "auto scale"), 0, 0);
+  dt_accel_register_iop(self, FALSE, NC_("accel", "camera model"), 0, 0);
+  dt_accel_register_iop(self, FALSE, NC_("accel", "lens model"), 0, 0);
 }
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_lensfun_gui_data_t *g = (dt_iop_lensfun_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_button_iop(self, "find lens",
+                              GTK_WIDGET(g->find_lens_button));
+  dt_accel_connect_button_iop(self, "lens model",
+                              GTK_WIDGET(g->lens_model));
+  dt_accel_connect_button_iop(self, "camera model",
+                              GTK_WIDGET(g->camera_model));
+  dt_accel_connect_button_iop(self, "find camera",
+                              GTK_WIDGET(g->find_camera_button));
+  dt_accel_connect_button_iop(self, "auto scale",
+                              GTK_WIDGET(g->auto_scale_button));
+
+  dt_accel_connect_slider_iop(self, "scale", GTK_WIDGET(g->scale));
+  dt_accel_connect_slider_iop(self, "tca R", GTK_WIDGET(g->tca_r));
+  dt_accel_connect_slider_iop(self, "tca B", GTK_WIDGET(g->tca_b));
+}
+
 void
 process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *ivoid, void *ovoid, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
@@ -702,6 +722,7 @@ static void camera_set (dt_iop_module_t *self, const lfCamera *cam)
   if (!cam)
   {
     gtk_button_set_label(GTK_BUTTON(g->camera_model), "");
+    gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->camera_model))), PANGO_ELLIPSIZE_END);
     g_object_set(G_OBJECT(g->camera_model), "tooltip-text", "", (char *)NULL);
     return;
   }
@@ -721,6 +742,7 @@ static void camera_set (dt_iop_module_t *self, const lfCamera *cam)
     else
       fm = g_strdup_printf ("%s", model);
     gtk_button_set_label (GTK_BUTTON (g->camera_model), fm);
+    gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->camera_model))), PANGO_ELLIPSIZE_END);
     g_free (fm);
   }
 
@@ -984,6 +1006,7 @@ static void lens_set (dt_iop_module_t *self, const lfLens *lens)
   if (!lens)
   {
     gtk_button_set_label(GTK_BUTTON(g->lens_model), "");
+    gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->lens_model))), PANGO_ELLIPSIZE_END);
     g_object_set(G_OBJECT(g->lens_model), "tooltip-text", "", (char *)NULL);
     return;
   }
@@ -1000,6 +1023,7 @@ static void lens_set (dt_iop_module_t *self, const lfLens *lens)
     else
       fm = g_strdup_printf ("%s", model);
     gtk_button_set_label (GTK_BUTTON (g->lens_model), fm);
+    gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->lens_model))), PANGO_ELLIPSIZE_END);
     g_free (fm);
   }
 
@@ -1058,7 +1082,6 @@ static void lens_set (dt_iop_module_t *self, const lfLens *lens)
   g_signal_connect (G_OBJECT(g->cbe[0]), "changed",
                     G_CALLBACK(lens_comboentry_focal_update), self);
   GtkWidget* button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/reset lens from exif data");
   gtk_box_pack_start(GTK_BOX(g->lens_param_box), button, FALSE, FALSE, 0);
   g_object_set(G_OBJECT(button), "tooltip-text", _("reset from exif data"), (char *)NULL);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -1074,7 +1097,6 @@ static void lens_set (dt_iop_module_t *self, const lfLens *lens)
   g_signal_connect (G_OBJECT(g->cbe[1]), "changed",
                     G_CALLBACK(lens_comboentry_aperture_update), self);
   button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/reset camera from exif data");
   gtk_box_pack_start(GTK_BOX(g->lens_param_box), button, FALSE, FALSE, 0);
   g_object_set(G_OBJECT(button), "tooltip-text", _("reset from exif data"), (char *)NULL);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -1086,7 +1108,6 @@ static void lens_set (dt_iop_module_t *self, const lfLens *lens)
   g_signal_connect (G_OBJECT(g->cbe[2]), "changed",
                     G_CALLBACK(lens_comboentry_distance_update), self);
   button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/reset distance from exif data");
   gtk_box_pack_start(GTK_BOX(g->lens_param_box), button, FALSE, FALSE, 0);
   g_object_set(G_OBJECT(button), "tooltip-text", _("reset from exif data"), (char *)NULL);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -1308,36 +1329,36 @@ void gui_init(struct dt_iop_module_t *self)
   // camera selector
   GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
   g->camera_model = GTK_BUTTON(gtk_button_new());
-  gtk_button_set_accel(g->camera_model,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/camera model");
   dt_gui_key_accel_block_on_focus (GTK_WIDGET (g->camera_model));
   gtk_button_set_label(g->camera_model, self->dev->image->exif_model);
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->camera_model))), PANGO_ELLIPSIZE_END);
   g_signal_connect (G_OBJECT (g->camera_model), "clicked",
                     G_CALLBACK (camera_menusearch_clicked), self);
   gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(g->camera_model), TRUE, TRUE, 0);
   button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/find camera");
+  g->find_camera_button = button;
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
   g_object_set(G_OBJECT(button), "tooltip-text", _("find camera"), (char *)NULL);
   g_signal_connect (G_OBJECT (button), "clicked",
                     G_CALLBACK (camera_autosearch_clicked), self);
-  gtk_table_attach(GTK_TABLE(self->widget), hbox, 0, 2, 0, 1, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+  gtk_table_attach(GTK_TABLE(self->widget), hbox, 0, 2, 0, 1, GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
   // lens selector
   hbox = gtk_hbox_new(FALSE, 0);
   g->lens_model = GTK_BUTTON(gtk_button_new());
-  gtk_button_set_accel(g->lens_model,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/lens model");
   dt_gui_key_accel_block_on_focus (GTK_WIDGET (g->lens_model));
   gtk_button_set_label(g->lens_model, self->dev->image->exif_lens);
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->lens_model))), PANGO_ELLIPSIZE_END);
   g_signal_connect (G_OBJECT (g->lens_model), "clicked",
                     G_CALLBACK (lens_menusearch_clicked), self);
   gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(g->lens_model), TRUE, TRUE, 0);
   button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/find lens");
+  g->find_lens_button = GTK_WIDGET(button);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
   g_object_set(G_OBJECT(button), "tooltip-text", _("find lens"), (char *)NULL);
   g_signal_connect (G_OBJECT (button), "clicked",
                     G_CALLBACK (lens_autosearch_clicked), self);
-  gtk_table_attach(GTK_TABLE(self->widget), hbox, 0, 2, 1, 2, GTK_EXPAND|GTK_FILL, 0, 0, 0);
+  gtk_table_attach(GTK_TABLE(self->widget), hbox, 0, 2, 1, 2, GTK_SHRINK|GTK_FILL, 0, 0, 0);
 
 
   // lens properties
@@ -1383,12 +1404,11 @@ void gui_init(struct dt_iop_module_t *self)
   g_signal_connect (G_OBJECT (g->scale), "value-changed",
                     G_CALLBACK (scale_changed), self);
   dtgtk_slider_set_label(g->scale, _("scale"));
-  dtgtk_slider_set_accel(g->scale,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/scale");
   hbox = gtk_hbox_new(FALSE, 0);
   gtk_box_pack_start(GTK_BOX(hbox), GTK_WIDGET(g->scale), TRUE, TRUE, 0);
 
   button = dtgtk_button_new(dtgtk_cairo_paint_refresh, CPF_STYLE_FLAT);
-  dtgtk_button_set_accel(DTGTK_BUTTON(button),darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/auto scale");
+  g->auto_scale_button = GTK_WIDGET(button);
   gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
   g_object_set(G_OBJECT(button), "tooltip-text", _("auto scale"), (char *)NULL);
   g_signal_connect (G_OBJECT (button), "clicked",
@@ -1409,8 +1429,6 @@ void gui_init(struct dt_iop_module_t *self)
   g->tca_b = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.99, 1.01, 0.0001, p->tca_b, 5));
   dtgtk_slider_set_label(g->tca_r, _("tca R"));
   dtgtk_slider_set_label(g->tca_b, _("tca B"));
-  dtgtk_slider_set_accel(g->tca_r,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/tca R");
-  dtgtk_slider_set_accel(g->tca_b,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lens/tca B");
   gtk_table_attach(GTK_TABLE(self->widget), GTK_WIDGET(g->tca_r), 0, 2, 6, 7, GTK_EXPAND|GTK_FILL, 0, 0, 0);
   gtk_table_attach(GTK_TABLE(self->widget), GTK_WIDGET(g->tca_b), 0, 2, 7, 8, GTK_EXPAND|GTK_FILL, 0, 0, 0);
   g_signal_connect (G_OBJECT (g->tca_r), "value-changed", G_CALLBACK (tca_changed), self);
@@ -1428,6 +1446,8 @@ void gui_update(struct dt_iop_module_t *self)
   lfDatabase *dt_iop_lensfun_db = (lfDatabase *)self->data;
   gtk_button_set_label(g->camera_model, p->camera);
   gtk_button_set_label(g->lens_model, p->lens);
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->camera_model))), PANGO_ELLIPSIZE_END);
+  gtk_label_set_ellipsize(GTK_LABEL(gtk_bin_get_child(GTK_BIN(g->lens_model))), PANGO_ELLIPSIZE_END);
   gtk_combo_box_set_active(g->target_geom, p->target_geom - LF_UNKNOWN - 1);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(g->reverse), p->inverse);
   dtgtk_slider_set_value(g->tca_r, p->tca_r);
