@@ -29,6 +29,7 @@
 #include "develop/develop.h"
 #include "control/control.h"
 #include "control/conf.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "gui/draw.h"
 #include "gui/presets.h"
@@ -91,10 +92,19 @@ groups ()
 }
 
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lowlight/blue shift");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blue shift"));
 }
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_lowlight_gui_data_t *g =
+      (dt_iop_lowlight_gui_data_t*)self->gui_data;
+  dt_accel_connect_slider_iop(self, "blue shift",
+                              GTK_WIDGET(g->scale_blueness));
+}
+
 static float
 lookup(const float *lut, const float i)
 {
@@ -227,7 +237,7 @@ void cleanup(dt_iop_module_t *module)
   module->params = NULL;
 }
 
-void init_presets (dt_iop_module_t *self)
+void init_presets (dt_iop_module_so_t *self)
 {
   dt_iop_lowlight_params_t p;
 
@@ -248,7 +258,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 0.0f;
-  dt_gui_presets_add_generic(_("daylight"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("daylight"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -265,7 +275,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 30.0f;
-  dt_gui_presets_add_generic(_("indoor bright"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("indoor bright"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -282,7 +292,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 30.0f;
-  dt_gui_presets_add_generic(_("indoor dim"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("indoor dim"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -299,7 +309,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 40.0f;
-  dt_gui_presets_add_generic(_("indoor dark"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("indoor dark"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -316,7 +326,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 50.0f;
-  dt_gui_presets_add_generic(_("twilight"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("twilight"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -333,7 +343,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 30.0f;
-  dt_gui_presets_add_generic(_("night street lit"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("night street lit"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -350,7 +360,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 30.0f;
-  dt_gui_presets_add_generic(_("night street"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("night street"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.150000;
@@ -367,7 +377,7 @@ void init_presets (dt_iop_module_t *self)
   p.transition_y[5] = 1.000000;
 
   p.blueness = 40.0f;
-  dt_gui_presets_add_generic(_("night street dark"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("night street dark"), self->op, self->version(), &p, sizeof(p), 1);
 
   p.transition_x[0] = 0.000000;
   p.transition_x[1] = 0.200000;
@@ -385,7 +395,7 @@ void init_presets (dt_iop_module_t *self)
 
 
   p.blueness = 50.0f;
-  dt_gui_presets_add_generic(_("night"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("night"), self->op, self->version(), &p, sizeof(p), 1);
 
   DT_DEBUG_SQLITE3_EXEC(darktable.db, "commit", NULL, NULL, NULL);
 }
@@ -722,7 +732,6 @@ void gui_init(struct dt_iop_module_t *self)
   dtgtk_slider_set_label(c->scale_blueness,_("blue shift"));
   dtgtk_slider_set_unit(c->scale_blueness,"%");
   dtgtk_slider_set_format_type(c->scale_blueness,DARKTABLE_SLIDER_FORMAT_PERCENT);
-  dtgtk_slider_set_accel(c->scale_blueness,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/lowlight/blue shift");
   g_object_set(G_OBJECT(c->scale_blueness), "tooltip-text", _("blueness in shadows"), (char *)NULL);
 
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(c->scale_blueness), TRUE, TRUE, 5);
