@@ -21,10 +21,12 @@
 #include "develop/imageop.h"
 #include "dtgtk/slider.h"
 #include "control/control.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "common/opencl.h"
 #include <gtk/gtk.h>
 #include <stdlib.h>
+#include <xmmintrin.h>
 
 #define ROUNDUP(a, n)		((a) % (n) == 0 ? (a) : ((a) / (n) + 1) * (n))
 
@@ -66,11 +68,26 @@ groups ()
   return IOP_GROUP_CORRECT;
 }
 
-void init_key_accels()
+int
+flags ()
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/nlmeans/luma");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/nlmeans/chroma");
+  return IOP_FLAGS_SUPPORTS_BLENDING;
 }
+
+void init_key_accels(dt_iop_module_so_t *self)
+{
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "luma"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "chroma"));
+}
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_nlmeans_gui_data_t *g = (dt_iop_nlmeans_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_slider_iop(self, "luma", GTK_WIDGET(g->luma));
+  dt_accel_connect_slider_iop(self, "chroma", GTK_WIDGET(g->chroma));
+}
+
 /** modify regions of interest (optional, per pixel ops don't need this) */
 // void modify_roi_out(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, dt_iop_roi_t *roi_out, const dt_iop_roi_t *roi_in);
 // void modify_roi_in(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *piece, const dt_iop_roi_t *roi_out, dt_iop_roi_t *roi_in);
@@ -130,7 +147,7 @@ error:
 #endif
 
 /** process, all real work is done here. */
-void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, void *o, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
+void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *ivoid, void *ovoid, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
   // this is called for preview and full pipe separately, each with its own pixelpipe piece.
   // get our data struct:
@@ -142,42 +159,38 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
   if(P <= 1)
   {
     // nothing to do from this distance:
-    memcpy (o, i, sizeof(float)*4*roi_out->width*roi_out->height);
+    memcpy (ovoid, ivoid, sizeof(float)*4*roi_out->width*roi_out->height);
     return;
   }
 
   // adjust to Lab, make L more important
-  float max_L = 100.0f, max_C = 256.0f;
-  float nL = 1.0f/(d->luma*max_L), nC = 1.0f/(d->chroma*max_C);
+  // float max_L = 100.0f, max_C = 256.0f;
+  // float nL = 1.0f/(d->luma*max_L), nC = 1.0f/(d->chroma*max_C);
+  float max_L = 120.0f, max_C = 512.0f;
+  float nL = 1.0f/max_L, nC = 1.0f/max_C;
   const float norm2[4] = { nL*nL, nC*nC, nC*nC, 1.0f };
 
-#define SLIDING_WINDOW // brings down time from 15 secs to 3 secs on a core2 duo
-#ifdef SLIDING_WINDOW
   float *Sa = dt_alloc_align(64, sizeof(float)*roi_out->width*dt_get_num_threads());
-#else
-  float *S = dt_alloc_align(64, sizeof(float)*roi_out->width*roi_out->height);
-#endif
   // we want to sum up weights in col[3], so need to init to 0:
-  memset(o, 0x0, sizeof(float)*roi_out->width*roi_out->height*4);
+  memset(ovoid, 0x0, sizeof(float)*roi_out->width*roi_out->height*4);
 
   // for each shift vector
   for(int kj=-K;kj<=K;kj++)
   {
     for(int ki=-K;ki<=K;ki++)
     {
-#ifdef SLIDING_WINDOW
       int inited_slide = 0;
       // don't construct summed area tables but use sliding window! (applies to cpu version res < 1k only, or else we will add up errors)
       // do this in parallel with a little threading overhead. could parallelize the outer loops with a bit more memory
 #ifdef _OPENMP
-#  pragma omp parallel for schedule(static) default(none) firstprivate(inited_slide) shared(kj, ki, roi_out, roi_in, i, o, Sa)
+#  pragma omp parallel for schedule(static) default(none) firstprivate(inited_slide) shared(kj, ki, roi_out, roi_in, ivoid, ovoid, Sa)
 #endif
       for(int j=0; j<roi_out->height; j++)
       {
         if(j+kj < 0 || j+kj >= roi_out->height) continue;
         float *S = Sa + dt_get_thread_num() * roi_out->width;
-        const float *ins = ((float *)i) + 4*(roi_in->width *(j+kj) + ki);
-        float *out = ((float *)o) + 4*roi_out->width*j;
+        const float *ins = ((float *)ivoid) + 4*(roi_in->width *(j+kj) + ki);
+        float *out = ((float *)ovoid) + 4*roi_out->width*j;
 
         const int Pm = MIN(MIN(P, j+kj), j);
         const int PM = MIN(MIN(P, roi_out->height-1-j-kj), roi_out->height-1-j);
@@ -189,17 +202,15 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
           memset(S, 0x0, sizeof(float)*roi_out->width);
           for(int jj=-Pm;jj<=PM;jj++)
           {
-            float *s = S;
-            const float *inp  = ((float *)i) + 4* roi_in->width *(j+jj);
-            const float *inps = ((float *)i) + 4*(roi_in->width *(j+jj+kj) + ki);
-            for(int i=0; i<roi_out->width; i++)
+            int i = MAX(0, -ki);
+            float *s = S + i;
+            const float *inp  = ((float *)ivoid) + 4*i + 4* roi_in->width *(j+jj);
+            const float *inps = ((float *)ivoid) + 4*i + 4*(roi_in->width *(j+jj+kj) + ki);
+            const int last = roi_out->width + MIN(0, -ki);
+            for(; i<last; i++, inp+=4, inps+=4, s++)
             {
-              if(i+ki >= 0 && i+ki < roi_out->width)
-              {
-                for(int k=0;k<3;k++)
-                  s[0] += (inp[4*i + k] - inps[4*i + k])*(inp[4*i + k] - inps[4*i + k]) * norm2[k];
-              }
-              s++;
+              for(int k=0;k<3;k++)
+                s[0] += (inp[k] - inps[k])*(inp[k] - inps[k]) * norm2[k];
             }
           }
           // only reuse this if we had a full stripe
@@ -217,9 +228,8 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
             slide += s[P] - s[-P-1];
           if(i+ki >= 0 && i+ki < roi_out->width)
           {
-            const float w = gh(slide);
-            for(int k=0;k<3;k++) out[k] += ins[k] * w;
-            out[3] += w;
+            const __m128 iv = { ins[0], ins[1], ins[2], 1.0f };
+            _mm_store_ps(out, _mm_load_ps(out) + iv * _mm_set1_ps(gh(slide)));
           }
           s   ++;
           ins += 4;
@@ -228,132 +238,100 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
         if(inited_slide && j+P+1+MAX(0,kj) < roi_out->height)
         {
           // sliding window in j direction:
-          float *s = S;
-          const float *inp  = ((float *)i) + 4* roi_in->width *(j+P+1);
-          const float *inps = ((float *)i) + 4*(roi_in->width *(j+P+1+kj) + ki);
-          const float *inm  = ((float *)i) + 4* roi_in->width *(j-P);
-          const float *inms = ((float *)i) + 4*(roi_in->width *(j-P+kj) + ki);
-          for(int i=0; i<roi_out->width; i++)
+          int i = MAX(0, -ki);
+          float *s = S + i;
+          const float *inp  = ((float *)ivoid) + 4*i + 4* roi_in->width *(j+P+1);
+          const float *inps = ((float *)ivoid) + 4*i + 4*(roi_in->width *(j+P+1+kj) + ki);
+          const float *inm  = ((float *)ivoid) + 4*i + 4* roi_in->width *(j-P);
+          const float *inms = ((float *)ivoid) + 4*i + 4*(roi_in->width *(j-P+kj) + ki);
+          const int last = roi_out->width + MIN(0, -ki);
+          for(; ((unsigned long)s & 0xf) != 0 && i<last; i++, inp+=4, inps+=4, inm+=4, inms+=4, s++)
           {
-            if(i+ki >= 0 && i+ki < roi_out->width) for(int k=0;k<3;k++)
-              s[0] += ((inp[4*i + k] - inps[4*i + k])*(inp[4*i + k] - inps[4*i + k])
-                    -  (inm[4*i + k] - inms[4*i + k])*(inm[4*i + k] - inms[4*i + k])) * norm2[k];
-            s++;
+            float stmp = s[0];
+            for(int k=0;k<3;k++)
+              stmp += ((inp[k] - inps[k])*(inp[k] - inps[k])
+                    -  (inm[k] - inms[k])*(inm[k] - inms[k])) * norm2[k];
+            s[0] = stmp;
+          }
+          /* Process most of the line 4 pixels at a time */
+          for(; i<last-4; i+=4, inp+=16, inps+=16, inm+=16, inms+=16, s+=4)
+          {
+            __m128 sv = _mm_load_ps(s);
+            const __m128 inp1 = _mm_load_ps(inp)    - _mm_load_ps(inps);
+            const __m128 inp2 = _mm_load_ps(inp+4)  - _mm_load_ps(inps+4);
+            const __m128 inp3 = _mm_load_ps(inp+8)  - _mm_load_ps(inps+8);
+            const __m128 inp4 = _mm_load_ps(inp+12) - _mm_load_ps(inps+12);
+
+            const __m128 inp12lo = _mm_unpacklo_ps(inp1,inp2);
+            const __m128 inp34lo = _mm_unpacklo_ps(inp3,inp4);
+            const __m128 inp12hi = _mm_unpackhi_ps(inp1,inp2);
+            const __m128 inp34hi = _mm_unpackhi_ps(inp3,inp4);
+
+            const __m128 inpv0 = _mm_movelh_ps(inp12lo,inp34lo);
+            sv += inpv0*inpv0 * _mm_set1_ps(norm2[0]);
+
+            const __m128 inpv1 = _mm_movehl_ps(inp34lo,inp12lo);
+            sv += inpv1*inpv1 * _mm_set1_ps(norm2[1]);
+
+            const __m128 inpv2 = _mm_movelh_ps(inp12hi,inp34hi);
+            sv += inpv2*inpv2 * _mm_set1_ps(norm2[2]);
+
+            const __m128 inm1 = _mm_load_ps(inm)    - _mm_load_ps(inms);
+            const __m128 inm2 = _mm_load_ps(inm+4)  - _mm_load_ps(inms+4);
+            const __m128 inm3 = _mm_load_ps(inm+8)  - _mm_load_ps(inms+8);
+            const __m128 inm4 = _mm_load_ps(inm+12) - _mm_load_ps(inms+12);
+
+            const __m128 inm12lo = _mm_unpacklo_ps(inm1,inm2);
+            const __m128 inm34lo = _mm_unpacklo_ps(inm3,inm4);
+            const __m128 inm12hi = _mm_unpackhi_ps(inm1,inm2);
+            const __m128 inm34hi = _mm_unpackhi_ps(inm3,inm4);
+
+            const __m128 inmv0 = _mm_movelh_ps(inm12lo,inm34lo);
+            sv -= inmv0*inmv0 * _mm_set1_ps(norm2[0]);
+
+            const __m128 inmv1 = _mm_movehl_ps(inm34lo,inm12lo);
+            sv -= inmv1*inmv1 * _mm_set1_ps(norm2[1]);
+
+            const __m128 inmv2 = _mm_movelh_ps(inm12hi,inm34hi);
+            sv -= inmv2*inmv2 * _mm_set1_ps(norm2[2]);
+
+            _mm_store_ps(s, sv);
+          }
+          for(; i<last; i++, inp+=4, inps+=4, inm+=4, inms+=4, s++)
+          {
+            float stmp = s[0];
+            for(int k=0;k<3;k++)
+              stmp += ((inp[k] - inps[k])*(inp[k] - inps[k])
+                    -  (inm[k] - inms[k])*(inm[k] - inms[k])) * norm2[k];
+            s[0] = stmp;
           }
         }
         else inited_slide = 0;
       }
-#else
-      // construct summed area table of weights:
-#ifdef _OPENMP
-  #pragma omp parallel for default(none) schedule(static) shared(i,S,roi_in,roi_out,kj,ki)
-#endif
-      for(int j=0; j<roi_out->height; j++)
-      {
-        const float *in  = ((float *)i) + 4* roi_in->width * j;
-        const float *ins = ((float *)i) + 4*(roi_in->width *(j+kj) + ki);
-        float *out = ((float *)S) + roi_out->width*j;
-        if(j+kj < 0 || j+kj >= roi_out->height) memset(out, 0x0, sizeof(float)*roi_out->width);
-        else for(int i=0; i<roi_out->width; i++)
-        {
-          if(i+ki < 0 || i+ki >= roi_out->width) out[0] = 0.0f;
-          else
-          {
-            out[0]  = (in[0] - ins[0])*(in[0] - ins[0]) * norm2[0];
-            out[0] += (in[1] - ins[1])*(in[1] - ins[1]) * norm2[1];
-            out[0] += (in[2] - ins[2])*(in[2] - ins[2]) * norm2[2];
-          }
-          in  += 4;
-          ins += 4;
-          out ++;
-        }
-      }
-      // now sum up:
-      // horizontal phase:
-#ifdef _OPENMP
-  #pragma omp parallel for default(none) schedule(static) shared(S,roi_out)
-#endif
-      for(int j=0; j<roi_out->height; j++)
-      {
-        int stride = 1;
-        while(stride < roi_out->width)
-        {
-          float *out = ((float *)S) + roi_out->width*j;
-          for(int i=0;i<roi_out->width-stride;i++)
-          {
-            out[0] += out[stride];
-            out ++;
-          }
-          stride <<= 1;
-        }
-      }
-      // vertical phase:
-#ifdef _OPENMP
-  #pragma omp parallel for default(none) schedule(static) shared(S,roi_out)
-#endif
-      for(int i=0; i<roi_out->width; i++)
-      {
-        int stride = 1;
-        while(stride < roi_out->height)
-        {
-          float *out = S + i;
-          for(int j=0;j<roi_out->height-stride;j++)
-          {
-            out[0] += out[roi_out->width*stride];
-            out += roi_out->width;
-          }
-          stride <<= 1;
-        }
-      }
-      // now the denoising loop:
-#ifdef _OPENMP
-  #pragma omp parallel for default(none) schedule(static) shared(i,o,S,kj,ki,roi_in,roi_out)
-#endif
-      for(int j=0; j<roi_out->height; j++)
-      {
-        if(j+kj < 0 || j+kj >= roi_out->height) continue;
-        const float *in  = ((float *)i) + 4*(roi_in->width *(j+kj) + ki);
-        float *out = ((float *)o) + 4*roi_out->width*j;
-        const float *s = S + roi_out->width*j;
-        const int offy = MIN(j, MIN(roi_out->height - j - 1, P)) * roi_out->width;
-        for(int i=0; i<roi_out->width; i++)
-        {
-          if(i+ki >= 0 && i+ki < roi_out->width)
-          {
-            const int offx = MIN(i, MIN(roi_out->width - i - 1, P));
-            const float m1 = s[offx - offy], m2 = s[- offx + offy], p1 = s[offx + offy], p2 = s[- offx - offy];
-            const float w = gh(p1 + p2 - m1 - m2);
-            for(int k=0;k<3;k++) out[k] += in[k] * w;
-            out[3] += w;
-          }
-          s   ++;
-          in  += 4;
-          out += 4;
-        }
-      }
-#endif
     }
   }
-  // normalize:
+  // normalize and apply chroma/luma blending
+  // bias a bit towards higher values for low input values:
+  const __m128 weight = _mm_set_ps(1.0f, powf(d->chroma, 0.6), powf(d->chroma, 0.6), powf(d->luma, 0.6));
+  const __m128 invert = _mm_sub_ps(_mm_set1_ps(1.0f), weight);
 #ifdef _OPENMP
-  #pragma omp parallel for default(none) schedule(static) shared(o,roi_out)
+  #pragma omp parallel for default(none) schedule(static) shared(ovoid,ivoid,roi_out,d)
 #endif
   for(int j=0; j<roi_out->height; j++)
   {
-    float *out = ((float *)o) + 4*roi_out->width*j;
+    float *out = ((float *)ovoid) + 4*roi_out->width*j;
+    float *in  = ((float *)ivoid) + 4*roi_out->width*j;
     for(int i=0; i<roi_out->width; i++)
     {
-      for(int k=0;k<3;k++) out[k] *= 1.0f/out[3];
+      _mm_store_ps(out, _mm_add_ps(
+          _mm_mul_ps(_mm_load_ps(in),  invert),
+          _mm_mul_ps(_mm_load_ps(out), _mm_div_ps(weight, _mm_set1_ps(out[3])))));
       out += 4;
+      in  += 4;
     }
   }
-#ifdef SLIDING_WINDOW
   // free shared tmp memory:
   free(Sa);
-#else
-  // free the summed area table:
-  free(S);
-#endif
 }
 
 /** this will be called to init new defaults if a new image is loaded from film strip mode. */
@@ -364,7 +342,7 @@ void reload_defaults(dt_iop_module_t *module)
   // init defaults:
   dt_iop_nlmeans_params_t tmp = (dt_iop_nlmeans_params_t)
   {
-    0.5f, 0.5f
+    0.1f, 0.3f
   };
   memcpy(module->params, &tmp, sizeof(dt_iop_nlmeans_params_t));
   memcpy(module->default_params, &tmp, sizeof(dt_iop_nlmeans_params_t));
@@ -467,16 +445,14 @@ void gui_init     (dt_iop_module_t *self)
   // TODO: adjust defaults:
   g->luma   = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0f, 100.0f, 1., 50.f, 0));
   g->chroma = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR, 0.0f, 100.0f, 1., 50.f, 0));
-  dtgtk_slider_set_default_value(g->luma,   50.f);
-  dtgtk_slider_set_default_value(g->chroma, 50.f);
+  dtgtk_slider_set_default_value(g->luma,   10.f);
+  dtgtk_slider_set_default_value(g->chroma, 30.f);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->luma), TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(g->chroma), TRUE, TRUE, 0);
   dtgtk_slider_set_label(g->luma, _("luma"));
   dtgtk_slider_set_unit (g->luma, "%");
   dtgtk_slider_set_label(g->chroma, _("chroma"));
   dtgtk_slider_set_unit (g->chroma, "%");
-  dtgtk_slider_set_accel(g->luma,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/nlmeans/luma");
-  dtgtk_slider_set_accel(g->chroma,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/nlmeans/chroma");
   g_object_set (GTK_OBJECT(g->luma),   "tooltip-text", _("how much to smooth brightness"), (char *)NULL);
   g_object_set (GTK_OBJECT(g->chroma), "tooltip-text", _("how much to smooth colors"), (char *)NULL);
   g_signal_connect (G_OBJECT (g->luma),   "value-changed", G_CALLBACK (luma_callback),   self);
