@@ -28,6 +28,7 @@
 #include "iop/profile_gamma.h"
 #include "develop/develop.h"
 #include "control/control.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 
 DT_MODULE(1)
@@ -43,11 +44,23 @@ groups ()
   return IOP_GROUP_COLOR;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/profile_gamma/linear part");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/profile_gamma/gamma exponential part");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "linear part"));
+  dt_accel_register_slider_iop(self, FALSE,
+                               NC_("accel", "gamma exponential part"));
 }
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_profile_gamma_gui_data_t *g =
+      (dt_iop_profile_gamma_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_slider_iop(self, "linear part", GTK_WIDGET(g->scale1));
+  dt_accel_connect_slider_iop(self, "gamma exponential part",
+                              GTK_WIDGET(g->scale2));
+}
+
 void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void *i, void *o, const dt_iop_roi_t *roi_in, const dt_iop_roi_t *roi_out)
 {
   dt_iop_profile_gamma_data_t *d = (dt_iop_profile_gamma_data_t *)piece->data;
@@ -182,12 +195,10 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(vbox), TRUE, TRUE, 5);
 
   g->scale1 = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 1.0, 0.0001,p->linear,4));
-  dtgtk_slider_set_accel(g->scale1,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/profile_gamma/linear part");
   g_object_set (GTK_OBJECT(g->scale1), "tooltip-text", _("linear part"), (char *)NULL);
   dtgtk_slider_set_label(g->scale1,_("linear"));
   g->scale2 = DTGTK_SLIDER(dtgtk_slider_new_with_range(DARKTABLE_SLIDER_BAR,0.0, 1.0, 0.0001,p->gamma,4));
   g_object_set (GTK_OBJECT(g->scale2), "tooltip-text", _("gamma exponential factor"), (char *)NULL);
-  dtgtk_slider_set_accel(g->scale2,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/profile_gamma/gamma exponential part");
   dtgtk_slider_set_label(g->scale2,_("gamma"));
   gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->scale1), TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->scale2), TRUE, TRUE, 0);
