@@ -18,6 +18,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <xmmintrin.h>
 #include <stdlib.h>
 #include <math.h>
 #include <assert.h>
@@ -29,6 +30,7 @@
 #include "control/control.h"
 #include "common/colorspaces.h"
 #include "common/opencl.h"
+#include "gui/accelerators.h"
 #include "gui/gtk.h"
 #include "libraw/libraw.h"
 #include "iop/wb_presets.c"
@@ -101,15 +103,30 @@ flags ()
   return IOP_FLAGS_ALLOW_TILING;
 }
 
-void init_key_accels()
+void init_key_accels(dt_iop_module_so_t *self)
 {
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/tint");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/temperature in");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/temperature out");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/red");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/green");
-  dtgtk_slider_init_accel(darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/blue");
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "tint"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "temperature in"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "temperature out"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "red"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "green"));
+  dt_accel_register_slider_iop(self, FALSE, NC_("accel", "blue"));
 }
+
+void connect_key_accels(dt_iop_module_t *self)
+{
+  dt_iop_temperature_gui_data_t *g =
+     (dt_iop_temperature_gui_data_t*)self->gui_data;
+
+  dt_accel_connect_slider_iop(self, "tint", GTK_WIDGET(g->scale_tint));
+  dt_accel_connect_slider_iop(self, "temperature in", GTK_WIDGET(g->scale_k));
+  dt_accel_connect_slider_iop(self, "temperature out",
+                              GTK_WIDGET(g->scale_k_out));
+  dt_accel_connect_slider_iop(self, "red", GTK_WIDGET(g->scale_r));
+  dt_accel_connect_slider_iop(self, "green", GTK_WIDGET(g->scale_g));
+  dt_accel_connect_slider_iop(self, "blue", GTK_WIDGET(g->scale_b));
+}
+
 int
 output_bpp(dt_iop_module_t *module, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
@@ -191,11 +208,30 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
 #endif
     for(int j=0; j<roi_out->height; j++)
     {
-      const uint16_t *in = ((uint16_t*)ivoid) + j*roi_out->width;
+      int i=0;
+      const uint16_t *in = ((uint16_t *)ivoid) + j*roi_out->width;
       float *out = ((float*)ovoid) + j*roi_out->width;
-      for(int i=0; i<roi_out->width; i++,out++,in++)
-        *out = *in * coeffsi[FC(j+roi_out->x, i+roi_out->y, filters)];
+
+      // process unaligned pixels
+      for ( ; i < ((4-(j*roi_out->width & 3)) & 3) ; i++,out++,in++)
+        *out = *in * coeffsi[FC(j+roi_out->y, i+roi_out->x, filters)];
+
+      const __m128 coeffs = _mm_set_ps(coeffsi[FC(j+roi_out->y, roi_out->x+i+3, filters)],
+                                       coeffsi[FC(j+roi_out->y, roi_out->x+i+2, filters)],
+                                       coeffsi[FC(j+roi_out->y, roi_out->x+i+1, filters)],
+                                       coeffsi[FC(j+roi_out->y, roi_out->x+i  , filters)]);
+
+      // process aligned pixels with SSE
+      for( ; i < roi_out->width - 3 ; i+=4,out+=4,in+=4)
+      {
+        _mm_stream_ps(out,_mm_mul_ps(coeffs,_mm_set_ps(in[3],in[2],in[1],in[0])));
+      }
+
+      // process the rest
+      for( ; i<roi_out->width; i++,out++,in++)
+        *out = *in * coeffsi[FC(j+roi_out->y, i+roi_out->x, filters)];
     }
+    _mm_sfence();
   }
   else if(piece->pipe->type != DT_DEV_PIXELPIPE_PREVIEW && filters && self->dev->image->bpp == 4)
   {
@@ -492,13 +528,6 @@ void gui_init (struct dt_iop_module_t *self)
   dtgtk_slider_set_label(g->scale_g,_("green"));
   dtgtk_slider_set_label(g->scale_b,_("blue"));
 
-  dtgtk_slider_set_accel(g->scale_tint,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/tint");
-  dtgtk_slider_set_accel(g->scale_k,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/temperature in");
-  dtgtk_slider_set_accel(g->scale_k_out,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/temperature out");
-  dtgtk_slider_set_accel(g->scale_r,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/red");
-  dtgtk_slider_set_accel(g->scale_g,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/green");
-  dtgtk_slider_set_accel(g->scale_b,darktable.control->accels_darkroom,"<Darktable>/darkroom/plugins/temperature/blue");
-
   gtk_box_pack_start(GTK_BOX(self->widget), GTK_WIDGET(vbox), TRUE, TRUE, 5);
   gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->scale_tint), TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(vbox), GTK_WIDGET(g->scale_k), TRUE, TRUE, 0);
