@@ -107,22 +107,26 @@ void process (struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, void
   {
     float *in = ((float *)i) + k*ch*roi_out->width;
     float *out = ((float *)o) + k*ch*roi_out->width;
+
+    const float low_approximation = d->table[(int)(0.01f * 0xfffful)];
+
     for (int j=0; j<roi_out->width; j++,in+=ch,out+=ch)
     {
       // in Lab: correct compressed Luminance for saturation:
       const float L_in = in[0]/100.0f;
-      out[0] = (L_in < 1.0f) ? d->table[CLAMP((int)(L_in*0xfffful), 0, 0xffff)] :
-        dt_iop_eval_exp(d->unbounded_coeffs, L_in);
-        
-      if(in[0] > 0.01f)
+      if(in[0] > 1.0f)
       {
-        out[1] = in[1] * out[0]/in[0];
-        out[2] = in[2] * out[0]/in[0];
+        out[0] = (L_in < 1.0f) ? d->table[CLAMP((int)(L_in*0xfffful), 0, 0xffff)] :
+          dt_iop_eval_exp(d->unbounded_coeffs, L_in);
+        
+          out[1] = in[1] * out[0]/in[0];
+          out[2] = in[2] * out[0]/in[0];
       }
       else
       {
-        out[1] = in[1] * out[0]/0.01f;
-        out[2] = in[2] * out[0]/0.01f;
+        out[0] = in[0] * low_approximation;
+        out[1] = in[1] * low_approximation;
+        out[2] = in[2] * low_approximation;
       }
     }
   }
@@ -136,7 +140,17 @@ void init_presets (dt_iop_module_so_t *self)
   float linear[6] = {0.0, 0.08, 0.4, 0.6, 0.92, 1.0};
   for(int k=0; k<6; k++) p.tonecurve_x[k] = linear[k];
   for(int k=0; k<6; k++) p.tonecurve_y[k] = linear[k];
-  dt_gui_presets_add_generic(_("linear"), self->op, &p, sizeof(p), 1);
+  p.tonecurve_y[1] += 0.03;
+  p.tonecurve_y[4] -= 0.03;
+  p.tonecurve_y[2] += 0.03;
+  p.tonecurve_y[3] -= 0.03;
+  for(int k=1; k<5; k++) p.tonecurve_y[k] = powf(p.tonecurve_y[k], 2.2f);
+  for(int k=1; k<5; k++) p.tonecurve_x[k] = powf(p.tonecurve_x[k], 2.2f);
+  dt_gui_presets_add_generic(_("low contrast"), self->op, self->version(), &p, sizeof(p), 1);
+
+  for(int k=0; k<6; k++) p.tonecurve_x[k] = linear[k];
+  for(int k=0; k<6; k++) p.tonecurve_y[k] = linear[k];
+  dt_gui_presets_add_generic(_("linear"), self->op, self->version(), &p, sizeof(p), 1);
 
   for(int k=0; k<6; k++) p.tonecurve_x[k] = linear[k];
   for(int k=0; k<6; k++) p.tonecurve_y[k] = linear[k];
@@ -146,7 +160,7 @@ void init_presets (dt_iop_module_so_t *self)
   p.tonecurve_y[3] += 0.03;
   for(int k=1; k<5; k++) p.tonecurve_y[k] = powf(p.tonecurve_y[k], 2.2f);
   for(int k=1; k<5; k++) p.tonecurve_x[k] = powf(p.tonecurve_x[k], 2.2f);
-  dt_gui_presets_add_generic(_("med contrast"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("med contrast"), self->op, self->version(), &p, sizeof(p), 1);
 
   for(int k=0; k<6; k++) p.tonecurve_x[k] = linear[k];
   for(int k=0; k<6; k++) p.tonecurve_y[k] = linear[k];
@@ -156,7 +170,7 @@ void init_presets (dt_iop_module_so_t *self)
   p.tonecurve_y[3] += 0.10;
   for(int k=1; k<5; k++) p.tonecurve_y[k] = powf(p.tonecurve_y[k], 2.2f);
   for(int k=1; k<5; k++) p.tonecurve_x[k] = powf(p.tonecurve_x[k], 2.2f);
-  dt_gui_presets_add_generic(_("high contrast"), self->op, &p, sizeof(p), 1);
+  dt_gui_presets_add_generic(_("high contrast"), self->op, self->version(), &p, sizeof(p), 1);
 }
 
 void commit_params (struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
