@@ -55,6 +55,8 @@ typedef struct dt_lib_module_t
   /** expander containing the widget. */
   GtkExpander *expander;
 
+  /** version */
+  int (*version)          ();
   /** get name of the module, to be translated. */
   const char* (*name)     ();
   /** get the views which the module should be loaded in. */
@@ -82,8 +84,13 @@ typedef struct dt_lib_module_t
   void* (*get_params)     (struct dt_lib_module_t *self, int *size);
   int   (*set_params)     (struct dt_lib_module_t *self, const void *params, int size);
   void  (*init_presets)   (struct dt_lib_module_t *self);
-  /** Optional callback for keyboard accelerators */
-  void (*init_key_accels)();
+  /** Optional callbacks for keyboard accelerators */
+  void (*init_key_accels)(struct dt_lib_module_t *self);
+  void (*connect_key_accels)(struct dt_lib_module_t *self);
+
+  GSList *accel_closures;
+  GtkWidget *reset_button;
+  GtkWidget *presets_button;
 }
 dt_lib_module_t;
 
@@ -96,11 +103,13 @@ int dt_lib_load_modules();
 void dt_lib_unload_module(dt_lib_module_t *module);
 /** creates a label widget for the expander, with callback to enable/disable this module. */
 GtkWidget *dt_lib_gui_get_expander(dt_lib_module_t *module);
+/** connects the reset and presets shortcuts to a lib */
+void dt_lib_connect_common_accels(dt_lib_module_t *module);
 
 
 /** preset stuff for lib */
 
 /** add or replace a preset for this operation. */
-void dt_lib_presets_add(const char *name, const char *plugin_name, const void *params, const int32_t params_size);
+void dt_lib_presets_add(const char *name, const char *plugin_name, const int32_t version, const void *params, const int32_t params_size);
 
 #endif
